#!/usr/bin/env python2
#
# Genny - xml DTD generation system
# Copyright (C) 2001-2004  James C. McDonald
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the 
#    Free Software Foundation, Inc.
#    59 Temple Place - Suite 330
#    Boston, MA  02111-1307, USA.
#
# created by:  Jim C. McDonald
#              McDonald Computer Corp.

#
#
# 04-19-2002 improved the layout of the table to put the xml tag name
#            on its own row with a spanspec, much more readable now
# 04-22-2002 using the enhanced XML LDD
# 07-22-2002 for v2.2 format
# 12-04-2002 for consolidated dir structure, take the input filename as argv[1]
# 12-04-2002 changed for an improved dir structure that allows for multiple
#            projects
# 02-06-2003 minor fixes for cross platform execution
# 07-25-2003 output the Short Name
# 02-04-2004 don't output the Short Name for Data Item Change

import xml.dom.minidom 
import sys
sys.path.append("./imports/")

import time
import string
import operator
import mismoextensions
import genny_util

__version__ = "0.15"

def getText(nodelist):
    rc = ""
    for node in nodelist:
        if node.nodeType == node.TEXT_NODE:
            rc = rc + node.data
    return rc

def hasChildByName(nodelist,inname):
    hit = 0
    for node in nodelist:
        if node.nodeName == inname:
            hit = 1
    return hit

if __name__ == '__main__':

    data_item_enum_cnt = 0
    data_item_enum_string_cnt = 0
    data_item_cnt = 0

    if len(sys.argv )< 3:
        print "Usage: python genny.py [project name] [filename]"
        sys.exit()
        
    # setup the filenames and paths based on the options passed to the script
    try:
        project_name = sys.argv[1]
        input_name = sys.argv[2]
        projdir = "./projects/" 
        outputfiledir         = projdir + project_name + "/output/"
        inputfiledir          = projdir + project_name + "/input/"
        inputcomponentdir    = inputfiledir + "docbook-components/"
        outputfilename       = outputfiledir + "chapter3.xml"
        logfilename          = outputfiledir + "logs/gennyLDD-to-docbook.log"
        infilename = inputfiledir + input_name + ".xml"
    except:
        print "ERROR !! Usage: python genny.py [project name] [filename]"
        sys.exit()

    logfh = open(logfilename,"w")    # open the logfile  
    outfh = open(outputfilename,"w") # open the outputfile

    currtime = time.time() # get the current time
    outfh.write("<!-- ############################################################### -->\n")
    outfh.write("<!--Generated by MISMO GennyLDD-to-docbook v%s on %s -->\n" % (__version__,(time.ctime(currtime))))
    outfh.write("""
<chapter id="ContainerDesc" >
 <title>Container Descriptions</title>
    <abstract>
        <para> This section of the document provides detailed information about the structure semantics of each container in the transaction.
        </para>
    </abstract>""")

##    outfh.write('<appendix>\n')
##    outfh.write("<title>Servicing Transfer Transaction Logical Data Dictionary</title>\n" )
##    outfh.write('<para></para>')

        
    print "\nStarting Parse ==="

    dom1 = xml.dom.minidom.parse(infilename)
    ldd = dom1.getElementsByTagName("GENNY_LDD")

    # not necessarily using these items right now
    transaction_schema_nodes = dom1.getElementsByTagName("TRANSACTION_SCHEMA")
    transaction_schema_id = transaction_schema_nodes[0].getAttribute("Id")
    transaction_schema_processarea = transaction_schema_nodes[0].getAttribute("ProcessArea")
    
    # print transaction_schema_id
    for node in transaction_schema_nodes[0].childNodes:

        if node.nodeName == "CONTAINER":
            enum_hit = 0

            container_id = node.getAttribute("Id")
            container_id  = string.replace(container_id,"Container ","")
            container_required = node.getAttribute("Required")
            container_xmlname = node.getAttribute("XMLName")
            container_definition = node.getAttribute("Definition")
            container_xmlparent = node.getAttribute("XMLParent")
            container_xmlsequence = node.getAttribute("XMLSequence")
            container_xmlminoccurs = node.getAttribute("XMLMinOccurs")
            container_xmlmaxoccurs = node.getAttribute("XMLMaxOccurs")
            container_implementationnote = node.getAttribute("ImplementationNote")
            container_example = node.getAttribute("Example")



            outfh.write('\n<section id="sectionid%s">\n' % (container_xmlname) )
            outfh.write("<title>%s<indexterm><primary>%s</primary></indexterm></title>\n" % (container_id,container_id) )

            outfh.write(' <formalpara>\n')
            outfh.write(' <title>Description</title>\n' )
            outfh.write(' <para>\n')
            outfh.write(' Information specific to the %s container.\n' % (container_id))
            outfh.write(' </para>\n')
            outfh.write(' </formalpara>\n')

            outfh.write(' <formalpara>\n')
            outfh.write('  <title>%s container structure</title>\n' % (container_id) )
            outfh.write('  <para>\n')
            outfh.write('   <mediaobject>\n')
            outfh.write('    <imageobject>\n')
            jpegname = string.replace(container_id," ","")
            outfh.write('     <imagedata align="center" fileref="./images/%s.jpg" format="JPG" />\n' % (jpegname) )
            outfh.write('    </imageobject>\n')            
            outfh.write('    <textobject>\n')   
            outfh.write('     <phrase>Graphical view of the structure of the %s container.</phrase>\n' % (container_id))
            outfh.write('    </textobject>\n')
            outfh.write('    <caption>\n')
            outfh.write('     <para>\n')
            outfh.write('      The structure of the %s container is shown above.\n' % (container_id))
            outfh.write('      </para>\n')
            outfh.write('    </caption>\n')
            outfh.write('   </mediaobject>\n')
            outfh.write('  </para>\n')
            outfh.write(' </formalpara>\n')

            outfh.write(' <table frame="all">\n' )
            outfh.write(" <title>%s Container Properties</title>\n" % (container_id) )
            outfh.write('  <tgroup cols="4" align="left" colsep="1" rowsep="1">\n' )
            outfh.write('  <colspec colname="ContainerName" />\n');
            outfh.write('  <colspec colname="ContainerDescription" />\n');
            outfh.write('  <colspec colname="ParentContainer" />\n');
            outfh.write('  <colspec colname="ChildrenContainer" />\n');
            outfh.write('   <thead>\n')
            outfh.write('    <row>\n')
            outfh.write('     <entry>Container XML Name</entry>\n')
            outfh.write('     <entry>Description</entry>\n')
            outfh.write('     <entry>Parent Container(s)</entry>\n')
            outfh.write('     <entry>Children Container(s)</entry>\n')
            outfh.write('    </row>\n')
            outfh.write('   </thead>\n')
            outfh.write('   <tbody>\n')
            outfh.write('    <row>\n')
            outfh.write('     <entry>%s</entry>\n' % (container_xmlname) )
            outfh.write('     <entry>%s</entry>\n' % (container_definition) )
            # output the list of parent(s)
            outfh.write('     <entry>')
            if hasChildByName(node.childNodes,"PARENT_CONTAINER"):
                for xnode in node.childNodes:
                    if xnode.nodeName == "PARENT_CONTAINER":
                        parent_container_id = xnode.getAttribute("Id")
                        if parent_container_id != 'Root':
                            outfh.write('<link linkend="sectionid%s">%s</link> '% (parent_container_id,parent_container_id))
                        else:
                            outfh.write('This is the root container for the transaction.')
            outfh.write('</entry>\n')
            # output the list of children 
            outfh.write('     <entry>')
            if hasChildByName(node.childNodes,"CHILD_CONTAINER"):
                for xnode in node.childNodes:
                    if xnode.nodeName == "CHILD_CONTAINER":
                        child_container_id = xnode.getAttribute("Id")
                        if child_container_id == "ANY":
                            pass
                        else:
                            outfh.write('<link linkend="sectionid%s">%s</link> '% (child_container_id,child_container_id))
            outfh.write('</entry>\n')
            #
            outfh.write('    </row>\n')
            outfh.write('   </tbody>\n')
            outfh.write("  </tgroup>\n" )                    
            outfh.write(" </table>\n" )



            # only ouput the data item table if we have data items and set a flag
            if hasChildByName(node.childNodes,"DATA_ITEM"):

                outfh.write(' <table frame="all">\n' )
                outfh.write(" <title>Data Item Properties for %s Container</title>\n" % (container_xmlname) )
                outfh.write('  <tgroup cols="4" align="left" colsep="1" rowsep="1">\n' )
                outfh.write('  <colspec colname="name" />\n');
                outfh.write('  <colspec colname="def" />\n');
                outfh.write('  <colspec colname="type" />\n');
                outfh.write('  <colspec colname="validval" />\n');
##                outfh.write('  <colspec colname="xmltagname" />\n');
                outfh.write('  <spanspec namest="def" nameend="validval" spanname="xmltagspan" />\n');

                outfh.write('   <thead>\n')
                outfh.write('    <row>\n')
                outfh.write('     <entry>Data Item Name</entry>\n')
                outfh.write('     <entry>Data Item Definition</entry>\n')
                outfh.write('     <entry>Data Item Data Type</entry>\n')
                outfh.write('     <entry>Valid Values and Definition</entry>\n')
##                outfh.write('     <entry>Data Item XML Tag Name</entry>\n')
                outfh.write('    </row>\n')
                outfh.write('   </thead>\n')

                outfh.write('   <tbody>\n')

                data_item_hit = 1
            else:
                data_item_hit = 0
            #
            for node in node.childNodes:
                if node.nodeName == "DATA_ITEM":
                    data_item_cnt += 1
                    data_item_id = node.getAttribute("Id")
                    data_item_required = node.getAttribute("Required")
                    data_item_xmlname = node.getAttribute("XMLName")
                    data_item_xmlshortname = node.getAttribute("XMLShortName")
                    data_item_xmldatatype = node.getAttribute("XMLDataType")
                    data_item_definition = node.getAttribute("Definition")
                    data_item_xmlsequence = node.getAttribute("XMLSequence")
                    data_item_xmlminoccurs = node.getAttribute("XMLMinOccurs")
                    data_item_xmlmaxoccurs = node.getAttribute("XMLMaxOccurs")
                    data_item_implementationnote = node.getAttribute("ImplementationNote")
                    data_item_example = node.getAttribute("Example")
                    data_item_usedbyprocess = node.getAttribute("UsedByProcess")
                    data_item_datamapnote = node.getAttribute("DataMappingNote")
                    data_item_source = node.getAttribute("SourceDocument")
                    data_item_idrefreference = node.getAttribute("IDREFReference")

                    data_item_enum_value = ""
                    data_item_enum_def = ""

                    if operator.mod(data_item_cnt,2) == 0:
                        mod_val = 0
                    else:
                        mod_val = 1 

                    outfh.write('    <row>\n')
                    outfh.write('     <entry>%s</entry>\n' % (string.replace(data_item_id, "DataItem ", "")) )
                    # 04-16-2004 changed the escaping of the definition to just the correct xml minimum encoding
                    #            and not the full set of extra stuff added in to handle the wierd characters in
                    #            the MISMO LDD repository
                    outfh.write('     <entry>%s</entry>\n' % (genny_util.xml_escape_lite(data_item_definition) ))
                    outfh.write('     <entry>%s</entry>\n' % (data_item_xmldatatype) )

                    #if we do not have enum values 
                    if not hasChildByName(node.childNodes,"DATA_ITEM_ENUM_VALUE"):
                        outfh.write('     <entry>%s</entry>\n' % (" ") )
                        outfh.write('    </row>\n')
                        outfh.write('    <row>\n')
                        outfh.write('     <entry></entry>\n')
                        if data_item_xmldatatype == "IDREF":
                            data_item_idrefreference = string.replace(data_item_idrefreference, "_", " ")
                            m_refname = mismoextensions.mismo_item_naming_scheme(data_item_idrefreference,container_xmlname,logfh,"Y")
                            outfh.write('     <entry spanname="xmltagspan">xml tag name: %s</entry>\n' % (data_item_xmlname +"_Referencing_"+ m_refname + "ID") )
                        elif data_item_xmldatatype == "IDREFS":
                            data_item_idrefreference = string.replace(data_item_idrefreference, "_", " ")
                            m_refname = mismoextensions.mismo_item_naming_scheme(data_item_idrefreference,container_xmlname,logfh,"Y")
                            outfh.write('     <entry spanname="xmltagspan">xml tag name: %s</entry>\n' % (data_item_xmlname +"_Referencing_"+ m_refname + "IDs"))
                        else:
                            # SHORTNAME TO FIX
                            if data_item_xmldatatype == 'ID':
                                data_item_xmlshortname = 'ID'
                            else:
                                    data_item_xmlshortname = ''
                            if len(data_item_xmlshortname) > 0 and project_name != 'DATA-ITEM-CHANGE':
                                outfh.write('     <entry spanname="xmltagspan">xml tag name:%s short name:%s</entry>\n' % (data_item_xmlname,data_item_xmlshortname) )
                            else:
                                outfh.write('     <entry spanname="xmltagspan">xml tag name: %s</entry>\n' % (data_item_xmlname) )
                        outfh.write('    </row>\n')
                    # if we have enum values then handle them 
                    else :
                        outfh.write('     <entry>\n') 
                        outfh.write('     <para>\n') 
                        outfh.write('     <itemizedlist>\n') 
                        for node in node.childNodes:
                            if node.nodeName == "DATA_ITEM_ENUM_VALUE":
                                data_item_enum_cnt += 1 
                                data_item_enum_value = node.getAttribute("Value")
                                data_item_enum_def = node.getAttribute("Definition")

                                if data_item_enum_value== "" : data_item_enum_value = "&nbsp;&nbsp;&nbsp;"
                                outfh.write("     <listitem><para>%s : " % (data_item_enum_value) )
                                if data_item_enum_def == "": data_item_enum_def = "No Definition provided."
                                outfh.write("%s</para></listitem>\n" % (genny_util.xml_escape_lite(data_item_enum_def)) )
                                data_item_enum_value = ""
                                data_item_enum_def = ""
                            elif node.nodeName == "DATA_ITEM_ENUM_DTD_STRING":
                                data_item_enum_string_cnt += 1 
                                data_item_enum_string_value = node.getAttribute("Value")
                        outfh.write('     </itemizedlist>\n') 
                        outfh.write('     </para>\n')
                        outfh.write('     </entry>\n') 
                        outfh.write('    </row>\n')
                        outfh.write('    <row>\n')
                        outfh.write('     <entry></entry>\n')
                        outfh.write('     <entry spanname="xmltagspan">xml tag name: %s</entry>\n' % (data_item_xmlname) )
                        outfh.write('    </row>\n')

            if (data_item_hit == 1):
                outfh.write('   </tbody>\n')
                outfh.write("  </tgroup>\n" )                    
                outfh.write(" </table>\n" )

# whole section to be removed when the stuff exists
##            tempinput = open("v22comptemplate.txt","r")  
##            temptext = tempinput.readlines()
##            tempfh = open(inputcomponentdir+container_xmlname+".txt","w")     
##            for templine in temptext:
##                tempfh.write(templine)
##            tempfh.close()
##            tempinput.close()

# 07-22-2002 open each of the component text files and add them to the end
# of the section for each container
#
            try:
                componentfh = open(inputcomponentdir+container_xmlname+".txt","r")
                #print ("Adding text for %s." % container_xmlname)
                componenttext = componentfh.readlines()
                for componentline in componenttext:
                    outfh.write(componentline)
            except:
                print ("Missing text for Chapter for %s component!" % container_xmlname)
                print ("Skipping %s component." % (container_xmlname))
            

##            # Implementation Notes Paragraph
##            outfh.write(' <formalpara>\n')
##            outfh.write('  <title>Implementation Notes, Special Considerations, Integration Issues, etc.</title>\n' )
##            outfh.write('  <para>\n')
##            outfh.write('   <itemizedlist>\n')
##            outfh.write('    <listitem>\n')
##            outfh.write('     <para>\n')
##            outfh.write('      Placeholder for the items for the %s container.\n' % (container_id))
##            outfh.write('      </para>\n')
##            outfh.write('    </listitem>\n')
##            outfh.write('   </itemizedlist>\n')
##            outfh.write('  </para>\n')
##            outfh.write(' </formalpara>\n')
##
##            # Examples Paragraph
##            outfh.write(' <formalpara>\n')
##            outfh.write('  <title>XML Examples</title>\n' )
##            outfh.write('  <para>\n')
##            outfh.write('      Below are XML examples for the %s container.\n' % (container_id))
##            outfh.write('  </para>\n')
##            outfh.write(' </formalpara>\n')
##
##            # Examples Paragraph
##            outfh.write(' <formalpara>\n')
##            outfh.write('  <title>Example 1</title>\n' )
##            outfh.write('  <para>\n')
##            outfh.write('      Placeholder for the 1st XML example for %s container.\n' % (container_id))
##            outfh.write('  </para>\n')
##            outfh.write(' </formalpara>\n')
            
            outfh.write('</section>\n')

##    outfh.write('</appendix>\n')
    outfh.write('</chapter>\n')
    outfh.write("<!-- ############################################################### -->\n")
    outfh.close()
    dom1.unlink()
    logfh.close()
    print ""
    print "%5i data items processed." % (data_item_cnt)
    print "%5i data item valid values processed." % (data_item_enum_cnt)
    print "%5i enum strings created." % (data_item_enum_string_cnt)
    print "%5i total items processed." % (data_item_cnt+data_item_enum_cnt)
    print ""
    print "Parse Completed ==="




