#!/usr/bin/env python2
#
# Genny - XML DTD generation system
# Copyright (C) 2001-2004  James C. McDonald
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the 
#    Free Software Foundation, Inc.
#    59 Temple Place - Suite 330
#    Boston, MA  02111-1307, USA.
#
# created by:  Jim C. McDonald
#              McDonald Computer Corp.

#
# 2002-11-18 v1.13 modified with new paths for input and output
# 2002-12-04 v1.14 modified to get inputfilename as sys.argv[2]
#                  and use the input filename as the outputfilename
# 2003-02-06 v1.15 minor fixes for cross platform execution
# 2003-02-12 v1.16 fixed hard coding of Servicing Transfer name in Title(s)
# 2004-05-26 v1.16 modified version to 2.3.1

import xml.dom.minidom 
import sys
sys.path.append("./imports/")

import time
import string
import operator
import mismoextensions
import genny_util

__version__ = "1.17"

def getText(nodelist):
    rc = ""
    for node in nodelist:
        if node.nodeType == node.TEXT_NODE:
            rc = rc + node.data
    return rc

def hasChildByName(nodelist,inname):
    hit = 0
    for node in nodelist:
        if node.nodeName == inname:
            hit = 1
    return hit

if __name__ == '__main__':

    data_item_enum_cnt = 0
    data_item_enum_string_cnt = 0
    data_item_cnt = 0

    if len(sys.argv )< 3:
        print "Usage: python genny.py [project name] [filename]"
        sys.exit()

    try:
        project_name = sys.argv[1]
        input_name = sys.argv[2]
        projdir = "./projects/" 
        outputfiledir         = projdir + project_name + "/output/"
        inputfiledir          = projdir + project_name + "/input/"

        infilename  = inputfiledir  + input_name + ".xml"
        outputfilename = outputfiledir + input_name + ".html"
        logfilename          = outputfiledir + "logs/gennyLDD-to-html.log"
    except:
        print "ERROR! Usage: python genny.py [project name] [filename]"
        sys.exit()

    logfh = open(logfilename,"w")    # open the logfile  
    outfh = open(outputfilename,"w") # open the outputfile
    outfh.write("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0 Transitional//EN\">\n")
    outfh.write("<!-- -->\n")

    currtime = time.time() # get the current time 
    outfh.write("<!--Generated by MISMO GennyLDD-to-html v%s on %s -->\n" % (__version__,(time.ctime(currtime))))

    outfh.write("<html>\n" )
    outfh.write("<head>\n" )
    outfh.write("<meta http-equiv=\"Content-Type\" content=\"text/html; charset=ISO-8859-1\" />\n" )
    outfh.write("<title>MISMO %s Version 2.3.1 LDD</title>\n" % project_name)
    outfh.write("</head>\n" )
    outfh.write("<body>\n" )

    outfh.write("<h1 align=\"center\">MISMO %s Version 2.3.1 LDD</h1>\n" % project_name)

    outfh.write("<p>\n")
    outfh.write(
"""
Copyright 2004 Mortgage Bankers Association of America, Inc.<br /><br /> 

Permission to use, copy, modify, and distribute the MISMO
DTD and its accompanying documentation for any purpose and without
fee is hereby granted in perpetuity, provided that the above copyright
notice and this paragraph appear in all copies. &nbsp;&nbsp; The copyright holders make
no representation about the suitability of the DTD for any purpose.
""")
    outfh.write("</p>\n")
    outfh.write("<hr />\n")
    
    print "\nStarting Parse ==="
    dom1 = xml.dom.minidom.parse(infilename)
    ldd = dom1.getElementsByTagName("GENNY_LDD")

    transaction_schema_nodes = dom1.getElementsByTagName("TRANSACTION_SCHEMA")
    transaction_schema_id = transaction_schema_nodes[0].getAttribute("Id")
    transaction_schema_processarea = transaction_schema_nodes[0].getAttribute("ProcessArea")

    outfh.write("<h1 align=\"center\">Transaction: %s</h1>\n" % (transaction_schema_id))
    outfh.write("<hr />\n") 


##    outfh.write("<table border=\"1\" summary=\"formatting\">\n")
##    outfh.write(" <tr bgcolor=\"#DCDCDC\">\n")
##    outfh.write("  <th align=\"center\">Transaction</th>\n")
##    outfh.write("  <th align=\"center\">Process Area</th>\n")
##    outfh.write(" </tr>\n")
##
##    outfh.write(" <tr bgcolor=\"#EEEEEE\">\n")
##    
##    outfh.write("  <td align=\"left\"><font size=\"3\">%s</font></td>\n" % (transaction_schema_id))
##    outfh.write("  <td align=\"left\"><font size=\"3\">%s</font></td>\n" % (transaction_schema_processarea))
##    outfh.write(" </tr>\n")
##    outfh.write("</table>\n")
##    outfh.write("<p>&nbsp;</p>\n")


##    outfh.write("<p align=\center\">Transaction: <strong>%s</strong><br /> Process Area: <strong>%s</strong><br /></p>\n" % (transaction_schema_id,transaction_schema_processarea))

   

    
    # print transaction_schema_id
    for node in transaction_schema_nodes[0].childNodes:

        if node.nodeName == "CONTAINER":
            enum_hit = 0

            container_id = node.getAttribute("Id")
            container_required = node.getAttribute("Required")
            container_xmlname = node.getAttribute("XMLName")
            container_definition = node.getAttribute("Definition")
            container_xmlparent = node.getAttribute("XMLParent")
            container_xmlsequence = node.getAttribute("XMLSequence")
            container_xmlminoccurs = node.getAttribute("XMLMinOccurs")
            container_xmlmaxoccurs = node.getAttribute("XMLMaxOccurs")
            container_implementationnote = node.getAttribute("ImplementationNote")
            container_example = node.getAttribute("Example")

            outfh.write("<h1 align=\"center\">Container: %s</h1>\n" % (string.replace(container_id, "Container ", "")))

            outfh.write("<table border=\"1\" width=\"100%\" summary=\"containerinfo\">\n")
##          outfh.write("<caption>data item info</caption>\n")
##          outfh.write(" <tr bgcolor=\"#DCDCDC\">\n")
            outfh.write(" <tr bgcolor=\"#a0b8c8\">\n")
            outfh.write("  <th align=\"center\">Container XML Name</th>\n")
##            outfh.write("  <th align=\"center\">Parent</th>\n")
            outfh.write("  <th align=\"center\">Description</th>\n")
            outfh.write(" </tr>\n")


##            outfh.write(" <tr bgcolor=\"#EEEEEE\">\n")
##            outfh.write("  <td align=\"right\"><font size=\"3\">Container Name:</font></td>\n" )
##            outfh.write("  <td align=\"left\"><font size=\"3\">%s</font></td>\n" % (container_id))
##            outfh.write("  </tr>\n")
##
##            if container_definition != "":
##                outfh.write(" <tr bgcolor=\"#EEEEEE\">\n")
##                outfh.write("  <td align=\"right\"><font size=\"3\">Container Description:</font></td>\n" )
##                outfh.write("  <td align=\"left\"><font size=\"3\">%s</font></td>\n" % (container_definition))
##                outfh.write("  </tr>\n")
##
##            outfh.write(" <tr bgcolor=\"#EEEEEE\">\n")
##            outfh.write("  <td align=\"right\"><font size=\"3\">Container Parent:</font></td>\n" )
##            outfh.write("  <td align=\"left\"><font size=\"3\">%s</font></td>\n" % (container_xmlparent))
##            outfh.write("  </tr>\n")

            if operator.mod(data_item_cnt,2) == 0:
#               outfh.write(" <tr bgcolor=\"#DCDCDC\">\n")
                outfh.write(" <tr bgcolor=\"#EEEEEE\">\n")
            else:
                outfh.write(" <tr bgcolor=\"#EEEEEE\">\n")
            
            outfh.write("  <td align=\"left\"><font size=\"3\">%s</font></td>\n" % (container_xmlname))
##            outfh.write("  <td align=\"left\"><font size=\"3\">%s</font></td>\n" % (container_xmlparent))
            if container_definition == "": container_definition = "&nbsp;"
            outfh.write("  <td align=\"left\"><font size=\"3\">%s</font></td>\n" % (container_definition))

            outfh.write(" </tr>\n")
            outfh.write("</table>\n")
            outfh.write("<p>&nbsp;</p>\n")

            # only ouput the data item table if we have data items and set a flag
            if hasChildByName(node.childNodes,"DATA_ITEM"):
                outfh.write("<table border=\"1\" width=\"100%\" summary=\"data item info\">\n")
##               outfh.write("<caption>data item info</caption>\n")
                outfh.write(" <tr bgcolor=\"#a0b8c8\">\n")
                outfh.write("  <th align=\"center\">Data Item</th>\n")

                outfh.write("  <th align=\"center\">Data Item<br />Definition</th>\n")
                outfh.write("  <th align=\"center\">Data Item<br />Datatype</th>\n")
                outfh.write("  <th align=\"center\">Valid Values<br />and Definition</th>\n")


                outfh.write("  <th align=\"center\">Data Item<br />XML Tag Name/Short Tag Name</th>\n")

                outfh.write(" </tr>\n")
                data_item_hit = 1
            else:
                data_item_hit = 0

            
            #print "container attrs #%s# #%s# #%s#" % (container_id,container_required,container_xmlname)
            #
            for node in node.childNodes:

                
                if node.nodeName == "DATA_ITEM":

                    data_item_cnt += 1
                    data_item_id = node.getAttribute("Id")
                    data_item_required = node.getAttribute("Required")
                    data_item_xmlname = node.getAttribute("XMLName")
                    data_item_xmlshortname = node.getAttribute("XMLShortName")
                    data_item_xmldatatype = node.getAttribute("XMLDataType")
                    data_item_definition = node.getAttribute("Definition")
                    data_item_definition = genny_util.html_escape(data_item_definition)
                    data_item_xmlsequence = node.getAttribute("XMLSequence")
                    data_item_xmlminoccurs = node.getAttribute("XMLMinOccurs")
                    data_item_xmlmaxoccurs = node.getAttribute("XMLMaxOccurs")
                    data_item_implementationnote = node.getAttribute("ImplementationNote")
                    data_item_example = node.getAttribute("Example")
                    data_item_usedbyprocess = node.getAttribute("UsedByProcess")
                    data_item_datamapnote = node.getAttribute("DataMappingNote")
                    data_item_source = node.getAttribute("SourceDocument")
                    data_item_idrefreference = node.getAttribute("IDREFReference")

                    #
                    # test_var = hasChildByName(node.childNodes,"DATA_ITEM_ENUM_VALUE")
                    # print "test_var %s" % str(test_var)

                    data_item_enum_value = ""
                    data_item_enum_def = ""

                    if operator.mod(data_item_cnt,2) == 0:
                        outfh.write(" <tr bgcolor=\"WHITE\">\n")
                    else:
                        outfh.write(" <tr bgcolor=\"#EEEEEE\">\n")

                    outfh.write("  <td align=\"left\"><font size=\"2\">%s</font></td>\n" % (string.replace(data_item_id, "DataItem ", "")) )
                    #outfh.write("  <td align=\"left\"><font size=\"2\">%s</font></td>\n" % (data_item_xmlname) )
                    outfh.write("  <td align=\"left\"><font size=\"2\">%s</font></td>\n" % (data_item_definition) )
                    outfh.write("  <td align=\"left\"><font size=\"2\">%s</font></td>\n" % (data_item_xmldatatype) )
                    
                    #if node.hasChildNodes() == 1:
                    if not hasChildByName(node.childNodes,"DATA_ITEM_ENUM_VALUE"):
                        outfh.write("  <td align=\"left\">&nbsp;</td>\n")
                        if data_item_xmldatatype == "IDREF":
                            ##outfh.write("  <td align=\"left\"><font size=\"2\">%s</font></td>\n" % (data_item_xmlname +"_Referencing"+container_xmlname+"ID"))
                            data_item_idrefreference = string.replace(data_item_idrefreference, "_", " ")
                            m_refname = mismoextensions.mismo_item_naming_scheme(data_item_idrefreference,container_xmlname,logfh,"Y")
                            print "IDREF: %s %s " % (data_item_xmlname,m_refname)
                            outfh.write("  <td align=\"left\"><font size=\"2\">%s</font></td>\n" % (data_item_xmlname +"_Referencing_"+ m_refname + "ID"))
                        elif data_item_xmldatatype == "IDREFS":
                            data_item_idrefreference = string.replace(data_item_idrefreference, "_", " ")
                            m_refname = mismoextensions.mismo_item_naming_scheme(data_item_idrefreference,container_xmlname,logfh,"Y")
                            print " %s %s " % (data_item_xmlname,m_refname)
                            outfh.write("  <td align=\"left\"><font size=\"2\">%s</font></td>\n" % (data_item_xmlname +"_Referencing_"+ m_refname + "IDs"))
                        else:
                            ## SHORTNAME FIX
                            if data_item_xmldatatype == 'ID':
                                data_item_xmlshortname = "ID"
                            elif data_item_xmldatatype == 'IDREF':
                                data_item_xmlshortname = ""
                            elif data_item_xmldatatype == 'IDREFS':
                                data_item_xmlshortname = ""
                            else:
                                if len(data_item_xmlshortname) == 0:
                                    data_item_xmlshortname = "D9999"
                                else:
                                    if data_item_xmlshortname[0] == 'D':
                                        # trim the D off the suggested name (if provided)
                                        data_item_xmlshortname = data_item_xmlshortname[1:]
                                    data_item_xmlshortname = ("D%04d" % int(data_item_xmlshortname))
                            #
                            if len(data_item_xmlshortname) > 0 and project_name != 'DATA-ITEM-CHANGE':
                                outfh.write("  <td align=\"left\"><font size=\"2\">%s / %s </font></td>\n" % (data_item_xmlname,data_item_xmlshortname) )
                            else:
                                outfh.write("  <td align=\"left\"><font size=\"2\">%s</font></td>\n" % (data_item_xmlname) )

                        outfh.write(" </tr>\n")
                    else :
                        outfh.write("  <td align=\"left\">\n    <table border=\"1\" summary=\"enum values\">\n")

                        for node in node.childNodes:
                            if node.nodeName == "DATA_ITEM_ENUM_VALUE":
                                data_item_enum_cnt += 1 
                                data_item_enum_value = node.getAttribute("Value")
                                data_item_enum_def = node.getAttribute("Definition")

                                outfh.write(" <tr>\n")
                                if data_item_enum_value== "" : data_item_enum_value = "&nbsp;&nbsp;&nbsp;"
                                outfh.write("  <td align=\"left\"><font size=\"2\">%s</font></td>\n" % (data_item_enum_value) )
                                if data_item_enum_def == "": data_item_enum_def = "No Definition for this value has been provided."
                                outfh.write("  <td align=\"left\"><font size=\"2\">%s</font></td>\n" % (data_item_enum_def) )
                                outfh.write(" </tr>\n")
                                data_item_enum_value = ""
                                data_item_enum_def = ""
                                 
                                #print "data item enum attrs #%s# #%s#" % (data_item_enum_value,data_item_enum_def)
                            elif node.nodeName == "DATA_ITEM_ENUM_DTD_STRING":
                                data_item_enum_string_cnt += 1 
                                data_item_enum_string_value = node.getAttribute("Value")
                                #print "data item enum string attrs #%s# " % (data_item_enum_string_value)
                                outfh.write("            </table>\n")
                                outfh.write("          </td>\n")
                                outfh.write("  <td align=\"left\"><font size=\"2\">%s</font></td>\n" % (data_item_xmlname) )
                                outfh.write("         </tr>\n")
##                                enum_hit = 1
##                if (enum_hit == 1 ):
##                    outfh.write("            </table>\n")
##                    outfh.write("          </td>\n")
##                    outfh.write("         </tr>\n")
##                    enum_hit == 0

            if (data_item_hit == 1):
                outfh.write("</table>\n")
                outfh.write("<p>&nbsp;</p>\n")
            outfh.write("<hr />\n")
            outfh.write("<p>&nbsp;</p>\n")
                    
    outfh.write("</body>\n" )
    outfh.write("</html>\n" )
    outfh.close()
    dom1.unlink()
    logfh.close()
    print ""
    print "%5i data items processed." % (data_item_cnt)
    print "%5i data item valid values processed." % (data_item_enum_cnt)
    print "%5i enum strings created." % (data_item_enum_string_cnt)
    print "%5i total items processed." % (data_item_cnt+data_item_enum_cnt)
    print ""
    print "Parse Completed ==="
   
"""
    #print "-->%s<--" % getText(ldd[0].childNodes)
    #
    #print ldd[0].childNodes

    items = dom1.getElementsByTagName("item")
    # print items[0].childNodes

    for node in items:
        containerx = node.childNodes[2]
        itemnamex = node.childNodes[5]
        datatypex = node.childNodes[8]
        print "----------------------------------------------------"
        print "container -->%s<--" % getText(containerx.childNodes)
        print "itemname  -->%s<--" % getText(itemnamex.childNodes)
        print "datatype  -->%s<--" % getText(datatypex.childNodes)
"""



