#!/usr/bin/env python2
#
# Genny - xml DTD generation system
# Copyright (C) 2001-2004  James C. McDonald
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the 
#    Free Software Foundation, Inc.
#    59 Temple Place - Suite 330
#    Boston, MA  02111-1307, USA.
#
# created by:  Jim C. McDonald
#              McDonald Computer Corp.
#
# This application was created to fill a need in the
# MISMO Servicing Workgroup
#
#
#  Changelog
# 06/14/2001 - started the conversion to the Genny LDD 
# 06/17/2001 - some refinements
# 06/18/2001 - the whole xml file is complete and well formed
# 06/19/2001 - parses as valid against the Genny LDD dtd (v.2)
# 06/20/2001 - changed to remove spaces from common valid values
# 06/21/2001 - update to v1.02 dtd
# 06/21/2001 - added counts for verification of processing
# 06/23/2001 - removed parenthesis from enumerated values
# 06/26/2001 - updates to v1.03 dtd
# 07/05/2001 - adding container description
# 07/09/2001 - additional datatype of Fixed, v1.04 dtd
# 07/15/2001 - upped dtd version to 1.05
# 07/16/2001 - back port updated mismo naming convention
# 07/18/2001 - back port of dtd version 1.05 attributes
# 07/18/2001 - dtd version 1.06
# 07/23/2001 - version .021, added ID, IDREF to valid
#              datatypes
# 08/06/2001 - remove parenthesis from enum values 
# 10/18/2001 - updated to draft v4
# 11/01/2001 - updated to v2.1 RC1
# 11/29/2001 - xml_escape/xml_unescape
# 01/04/2002 - updated to v2.1 RC 2
# 01/24/2002 - added handling for IDREFS,EnumeratedWithDefault, and
#              MISMO standard compatible handling of the naming for IDREF
#              attributes
# 02/05/2002 - v2.1 final
#
# 04/19/2002 - created as "enhanced" version after v2.1
# 04/22/2002 - the xml file validates against the 2.00 Genny LDD dtd
#
# 07/22/2002 - started work on v2.2
# 10/08/2002 - the xml file validates against the 2.01 Genny LDD dtd
#              changed to skip the first column and the header stuff
#              that excel puts in the tab delimited format by default
# 12/04/2002 - consolidated directory structure and simplified
# 02/13/2003 - corrected the GennyLDD version to v2.01
# 06/07/2003 - additional debugging statements
# 07/25/2003 - updated to the GennyLDD version v2.02 and
#              the data item short name
# 02/19/2004 - added the ability to add a set of containers that
#              is not in the original spreadsheet
# 07/26/2004 - enhanced the processing of enumerated values
#              to allow for data item names to be present on
#              each line in the valid values

import sys
sys.path.append("./imports/")
import ASV
import time
import string
import mismoextensions
import genny_util

__version__ = ".061"

# **************************************************************
# for tab delimited file format to Genny LDD xml
# generation
# **************************************************************

# **************************************************************
# Parameters
#


use_fq = "Y"

debug = 0

headertext="""<!-- ====================================================
     This is a Genny style Logical Data Dictionary from 
     which several items may be __generated__ (e.g. DTD,
     xsd Schema, etc). 
     ===================================================== -->
"""


def parse_LDD_file(filename,logfh):
    """Parse the Input LDD as a tab delimited file.

    The root container must be the first line after the column headers of the
    input file.  Everything prior the the line with the Root container is
    skipped. 
    """
    LDD_rows = ASV.ASV()
    print "Starting parse of LDD ..."
    logfh.write("Starting parse of LDD ...\n")
    LDD_rows.input_from_file(filename, ASV.TSV(), has_field_names = 0)
    print "Completed parse of LDD ..."
    logfh.write("Completed parse of LDD ...\n")
    
    # trim the first line and the fist column
    # the Root container must be the fist line after the column headers
    row_count = 0
    found_root = 0
    new_LDD_rows = []
    for row in LDD_rows:
        if len(row) >= 10:
            if string.upper(row[10]) == 'ROOT':
                found_root = 1
        if found_root == 1:
            new_row = row[1:]
            new_LDD_rows.append(new_row)
##            if row_count < 5: print new_row
        row_count = row_count + 1
    return new_LDD_rows


def log_LDD_rows(LDD_rows,logfh):
    """Writes the retrieved input rows to be converted to a log file
    """
    print "Starting logging input ..." 
    row_cnt = 0
    sys.stdout.write("Row count-->")
    for row in LDD_rows:
        # print row
        sys.stdout.write ("%5i<--\b\b\b\b\b\b\b\b" % (row_cnt+1))
        logfh.write("row# %4i: %-40s: %-40s: %-40s: %-40s: %-40s: %-40s: %-40s:\n" % (row_cnt,row[0],row[1],row[2],row[3],row[4],row[5],row[6]))
        row_cnt += 1
    print "\nCompleted logging of input ..."
    logfh.write("total number of data items processed = %i\n" % (row_cnt))
    


def xml_header(logfh,xmlfh):
    """ Start to build the LDD xml file, and include the boilerplate
    header text
    """
    print "Adding header text to GennyLDD ..."
    currtime = time.time() # get the current time 
    xmlfh.write("<?xml version='1.0' encoding='UTF-8' ?>\n")
    xmlfh.write("<!DOCTYPE GENNY_LDD SYSTEM \"gennyLDD-v202.dtd\">\n")
    xmlfh.write("<!--Generated by MISMO to GennyLDD v%s on %s -->\n" % (__version__,(time.ctime(currtime))))
##    bpfh = open(headerfilename,"r")
##    bptext = bpfh.readlines()
##    bpfh.close()
##    xmlfh.writelines(bptext)
    #
    xmlfh.write("%s" % headertext) 
    logfh.write("Added boilerplate text to GennyLDD ...\n")


def xml_body(logfh,xmlfh,LDD_rows,inputfiledir):
    """Write the xml that is the LDD to the output file
    """
    print "Adding structure text to dtd ..."
    xmlfh.write("<GENNY_LDD VersionId=\"2.02\">\n")
    xmlfh.write(" <TRANSACTION_SCHEMA\n  Id=\"%s\"\n  ProcessArea=\"%s\" >\n" % (transaction_schema,processareaname) )
    xml_containers(logfh,xmlfh,LDD_rows) # output the container elements of the xml file

    # handle the addition of extra containers not in the spreadsheet
    # 02-19-2004
    try:
        addonfh = open(inputfiledir+"ADD-TO-GENNYLDD.xml","r")
        addontext = addonfh.readlines()
        for addonline in addontext:
            xmlfh.write(addonline)
    except:
        print ("No extra containers to add.")
    
    xmlfh.write(" </TRANSACTION_SCHEMA>\n")
    xmlfh.write("</GENNY_LDD>\n")


def xml_containers(logfh,xmlfh,LDD_rows):
    """
    # **************************************************************
    # write the xml for each container that is found to the output
    # file
    #
    # **************************************************************
    # column map
    #       0   Container Name
    #       1   Container Description
    #       2   data item name (LDD name)
    #       3   Data Item Definition
    #       4   Common Valid Values
    #       5   Common Valid Values Definition
    #       6   Datatype
    #       7   IDREF container reference/Fixed/Default Value
    #
    #       8   container sequence
    #       9   parent containers (or Root)
    #       10  child containers (with min/maxOccurs for each)
    #       11  container default minOccurs
    #       12  container default maxOccurs
    #       13  container primary key item value
    #       14  container implementation note
    #       15  container example
    #       16  container used by process
    #       17  container doc graphic filename

    #       18  data item sequence
    #       19  data item minOccurs
    #       20  data item maxOccurs
    #       21  data item implementation note
    #       22  data item example
    #       23  data item used by process
    #       24  data item mapping note
    #       25  data item source document
    #       26  data item min length
    #       27  data item max length
    #       28  data item regex pattern
    #       29  data item retain whitespace indicator
    #       30  data item numeric precision
    #       31  data item numeric scale
    #       32  data item min value inclusive
    #       33  data item max value inclusive
    #       34  data item short name
    #
    #       35  "EOL" - strictly a placeholder so excel with save the columns correctly
    #
    # **************************************************************
    """
    print "Adding containers with to GennyLDD ..."
    row = LDD_rows[0]            # extract the first row
    prev_container_name = row[0] # prime the prev name logic
    container_cnt = 0            # prime the container count
    containers = []              # empty list for the containters
    container_rows = []          # empty list for the rows for a specific container
    data_item_enum_cnt = 0
    data_item_enum_string_cnt = 0
    data_item_cnt = 0
    i = 0
    while i < len(LDD_rows):
        row = LDD_rows[i]
        if prev_container_name == row[0] :   # keep collecting rows as long as they are the same as previous
            container_rows.append(row)
            logfh.write("   appending subelement %s\n" % (row[2]) )
        else :
            # handle end of a container
            print "  Processing %s container row %i" % (prev_container_name,i)
            logfh.write("Processing %s container\n" % (prev_container_name) )
            containers.append(prev_container_name) # store the container name
            prev_container_name = row[0]           # reset the prev container name
            container_cnt += 1
            #

            data_item_cnt,data_item_enum_cnt,data_item_enum_string_cnt = xml_write_container_attr(logfh,xmlfh,container_rows,data_item_cnt,data_item_enum_cnt,data_item_enum_string_cnt)
            #
            container_rows = [] # reset the container list back to zilch 
            container_rows.append(row) # the row that is not the same as the previous is the first in the list
            logfh.write("   appending subelement %s\n" % (row[2]) )
        i += 1  # index to the next row in the LDD
    if i == len(LDD_rows):
        print "  End Processing %s container row %i" % (prev_container_name,i)
        logfh.write("End Processing %s container\n" % (prev_container_name) )
        containers.append(prev_container_name) # store the container name
        prev_container_name = row[0] # reset the prev container name
        container_cnt += 1
        #
        data_item_cnt,data_item_enum_cnt,data_item_enum_string_cnt = xml_write_container_attr(logfh,xmlfh,container_rows,data_item_cnt,data_item_enum_cnt,data_item_enum_string_cnt)
        #
        container_rows = [] # reset the container list back to zilch 
        container_rows.append(row) # the row that is not the same as the previous is the first in the list
        logfh.write("   appending subelement %s\n" % (row[2]) )     
    print ""
    print "%5i containers have been processed." % (container_cnt)
    print "%5i data items processed." % (data_item_cnt)
    print "%5i data item valid values processed." % (data_item_enum_cnt)
    print "%5i enum strings created." % (data_item_enum_string_cnt)
    print "%5i total items processed." % (data_item_cnt+data_item_enum_cnt)
    print ""
    print "There should be %i rows in input file!" % (data_item_cnt+data_item_enum_cnt-data_item_enum_string_cnt) 

    logfh.write("%i containers have been processed.\n" % (container_cnt))


def parse_childlist(logfh,inputstring):
    """ Parse the list of child container/min/max occurs into discrete
    components
    """
    childlist = []
    child_rows = string.split(inputstring,",")
    for row in child_rows:
        new_row = []
        items = string.split(row,":")
        for item in items:
            new_row.append(item)
        childlist.append(new_row)
    return childlist
    

def xml_write_container_attr(logfh,xmlfh,container_rows,data_item_cnt,data_item_enum_cnt,data_item_enum_string_cnt):
    """ Output the xml for a specific container 
    """
    row = container_rows[0]
    # print container_rows
    xmlfh.write("<!-- ================================================================= -->\n")
    xmlfh.write("<!-- %-40s                          -->\n" % (string.upper(string.replace(row[0]," ","_"))))
    xmlfh.write("<!-- ================================================================= -->\n")

    # print the opening tag and container attributes
    xmlfh.write("  <CONTAINER \n" )
    xmlfh.write("   Id=\"Container %s\"  \n"      % (row[0]))
    xmlfh.write("   XMLName=\"%s\" \n"            % (string.upper(string.replace(row[0]," ","_"))) )
    xmlfh.write("   Definition=\"%s\" \n"         % (row[1]) )
    xmlfh.write('   ParentContainerList="%s"\n'   % (row[9]) )
    xmlfh.write('   ChildrenContainerList="%s"\n' % (row[10]))
    xmlfh.write("   XMLSequence=\"%s\" \n"        % (row[8]) )
    xmlfh.write("   DefaultMinOccurs=\"%s\" \n"       % (row[11]))
    xmlfh.write("   DefaultMaxOccurs=\"%s\" \n"       % (row[12]))
    xmlfh.write("   PrimaryKeyItem=\"%s\" \n"     % (row[13]))
    xmlfh.write("   ImplementationNote=\"%s\" \n" % (row[14]))
    xmlfh.write("   Example=\"%s\" \n"            % (row[15]))
    xmlfh.write("   UsedByProcessArea=\"%s\" \n"  % (row[16]))
    xmlfh.write('   DocGraphicFilename="%s" '     % (row[17]))
    xmlfh.write("   >\n")

    # output the Parent containers
    parentlist = string.split(row[9],",")
    for parent_name in parentlist:
        xmlfh.write('   <PARENT_CONTAINER Id="%s" />\n' % parent_name )

#   print "Debug %s" % row[10]
    # output the Child containers   and row[10] != "Root" 
    if row[10] != "" :
        childlist = parse_childlist(logfh,row[10])
        #print "Debug %s -%s" % (row[10],childlist)
        for child_row in childlist:
            xmlfh.write('   <CHILD_CONTAINER Id="%s"\n' % child_row[0] )
            xmlfh.write("    XMLMinOccurs=\"%s\" \n"    % child_row[1] )
            xmlfh.write("    XMLMaxOccurs=\"%s\" \n"    % child_row[2] )
            xmlfh.write("    />\n") 

    # print the xml for each data item within the container element
    i = 0
    while i < len(container_rows):
        row = container_rows[i]
        genny_datatype = ""
        genny_definition = genny_util.xml_escape(row[3]) 
        
        if row[2] != "":
            # map the datatype from the Input to the GennyLDD std types

            logfh.write("row %i name %s datatype -->%s<--\n" % (i,row[2],row[6]))           

            genny_datatype_fixedval = ""
            genny_datatype_idrefref = ""
            if row[6] == "enumerated":
                genny_datatype = "Enumerated"
            elif row[6] == "string":
                genny_datatype = "Alphanumeric"
            elif row[6] == "date/time":
                genny_datatype = "DateTime"
            elif row[6] == 'date':
                genny_datatype = "Date"
            elif row[6] == "money":
                genny_datatype = "Money"
            elif row[6] == "numeric":
                genny_datatype = "Numeric"
            elif row[6] == "boolean":
                genny_datatype = "Boolean"
            elif row[6] == "xsd:boolean":
                genny_datatype = "Boolean"
            elif row[6] == "fixed":
                genny_datatype = "Fixed"
                genny_datatype_fixedval = row[7]
                genny_datatype_idrefref = ""
            elif row[6] == "EnumeratedWithDefault":
                genny_datatype = "EnumeratedWithDefault"
                genny_datatype_fixedval = row[7]
                genny_datatype_idrefref = ""
            elif row[6] == "ID":
                genny_datatype = "ID"
            elif row[6] == "IDREF":
                genny_datatype = "IDREF"
                genny_datatype_fixedval = ""
                genny_datatype_idrefref = row[7]
            elif row[6] == "IDREFS":
                genny_datatype = "IDREFS"
                genny_datatype_fixedval = ""
                genny_datatype_idrefref = row[7]
            else:
                genny_datatype = "Undefined"

            data_item_cnt += 1
            xmlfh.write("    <DATA_ITEM \n" )
            xmlfh.write('     Id="DataItem %s" \n'                 % ( row[2] ) )
            xmlfh.write("     XMLName=\"%s\" \n" %  (mismoextensions.mismo_item_naming_scheme(row[2],row[0],logfh,use_fq)) )            
            xmlfh.write("     XMLDataType=\"%s\" \n"               % (genny_datatype) )
            xmlfh.write("     Definition=\"%s\" \n"                % (genny_definition))
            if row[18] == "":
                xmlfh.write("     XMLSequence=\"%i\" \n"           % (i+1))
            else:
                xmlfh.write("     XMLSequence=\"%i\" \n"           % row[18] )
            xmlfh.write("     XMLMinOccurs=\"%s\" \n"              % row[19] )
            xmlfh.write("     XMLMaxOccurs=\"%s\" \n"              % row[20])
            xmlfh.write("     ImplementationNote=\"%s\" \n"        % genny_util.xml_escape(row[21]) )
            xmlfh.write("     Example=\"%s\" \n"                   % genny_util.xml_escape(row[22]))
            xmlfh.write("     UsedByProcessArea=\"%s\" \n"         %  row[23] )
            xmlfh.write("     DataMappingNote=\"%s\" \n"           % genny_util.xml_escape(row[24]) )           
            xmlfh.write("     SourceDocument=\"%s\" \n"            % row[25] )  
            xmlfh.write("     IDREFReference=\"%s\" \n"            % ( genny_datatype_idrefref ) )
            xmlfh.write("     FixedValue=\"%s\" \n"                % ( genny_datatype_fixedval ) )
            xmlfh.write("     minLength=\"%s\" \n"                 % row[26] )  
            xmlfh.write("     maxLength=\"%s\" \n"                 % row[27] )  
            xmlfh.write("     regexPattern=\"%s\" \n"              % row[28] )
            if row[29] == "" :
                retain = "false"
            else:
                retain = row[29]
            xmlfh.write("     RetainWhitespaceIndicator=\"%s\" \n" % retain )   
            xmlfh.write("     numericPrecision=\"%s\" \n"          % row[30] )  
            xmlfh.write("     numericScale=\"%s\" \n"              % row[31] )  
            xmlfh.write("     minInclusive=\"%s\" \n"              % row[32] )
            xmlfh.write("     maxInclusive=\"%s\" \n"              % row[33] )  
            xmlfh.write("     XMLShortName=\"%s\" \n"              % row[34] )
            xmlfh.write("     >\n")
            
            if (genny_datatype == "Enumerated") or (genny_datatype == "EnumeratedWithDefault"):
                data_item_enum_cnt +=1
                # handle the enumerated values
                enumvaluestring = ""
                xmlfh.write("      <DATA_ITEM_ENUM_VALUE \n" )
                enum_value = string.replace(row[4]," ","")
                enum_value = string.replace(enum_value,"/","or")
                enum_value = string.replace(enum_value,"(","")
                enum_value = string.replace(enum_value,")","")
                enum_value = string.replace(enum_value,",","-")
                enum_value = string.replace(enum_value,":","-")
                enum_value = string.replace(enum_value,"'","")
                xmlfh.write("       Value=\"%s\" \n" % ( enum_value ) )
                genny_enum_definition = genny_util.xml_escape(row[5])
                xmlfh.write("       Definition=\"%s\" " % (genny_enum_definition) )
                xmlfh.write("/>\n")
                enumvaluestring = enumvaluestring + " | " + enum_value
                i += 1
                if i < len(container_rows):
                    row = container_rows[i] 
                # start processing the rest of the valid values
                # modified 07/26/2004 to allow for old and new styles
                # of data item entry to both work
                if row[2] == "":
                    # Handle the "old style" with no data item name entered
                    # i.e. loop until a data item name is found
                    while row[2] == "" and (i < len(container_rows)):
                        data_item_enum_cnt +=1
                        xmlfh.write("      <DATA_ITEM_ENUM_VALUE \n" )
                        enum_value = string.replace(row[4]," ","")
                        enum_value = string.replace(enum_value,"/","or")
                        enum_value = string.replace(enum_value,"(","")
                        enum_value = string.replace(enum_value,")","")
                        enum_value = string.replace(enum_value,",","-")
                        enum_value = string.replace(enum_value,":","-")
                        enum_value = string.replace(enum_value,"'","")
                        xmlfh.write("       Value=\"%s\" \n" % ( enum_value ) )
    
                        genny_enum_definition = genny_util.xml_escape(row[5])
                        xmlfh.write("       Definition=\"%s\" " % (genny_enum_definition) )
                        xmlfh.write("/>\n")
                        enumvaluestring = enumvaluestring + " | " + enum_value
                        i += 1
                        if i < len(container_rows):
                            row = container_rows[i]
                else:
                    # Handle the style with data item names entered for 
                    # each valid value
                    # i.e. loop until the data item name changes
                    current_data_item_name = row[2]
                    while row[2] == current_data_item_name and (i < len(container_rows)):
                        data_item_enum_cnt +=1
                        xmlfh.write("      <DATA_ITEM_ENUM_VALUE \n" )
                        enum_value = string.replace(row[4]," ","")
                        enum_value = string.replace(enum_value,"/","or")
                        enum_value = string.replace(enum_value,"(","")
                        enum_value = string.replace(enum_value,")","")
                        enum_value = string.replace(enum_value,",","-")
                        enum_value = string.replace(enum_value,":","-")
                        enum_value = string.replace(enum_value,"'","")
                        xmlfh.write("       Value=\"%s\" \n" % ( enum_value ) )
    
                        genny_enum_definition = genny_util.xml_escape(row[5])
                        xmlfh.write("       Definition=\"%s\" " % (genny_enum_definition) )
                        xmlfh.write("/>\n")
                        enumvaluestring = enumvaluestring + " | " + enum_value
                        i += 1
                        if i < len(container_rows):
                            row = container_rows[i]
                        
                # after we have processed all of enumerated values we
                # can output the whole dtd string
                data_item_enum_string_cnt += 1
                xmlfh.write("      <DATA_ITEM_ENUM_DTD_STRING\n" )
                # changed 01/24/2002
                #
                ## enumvaluestring = enumvaluestring[3:]  # slice off the first sp|sp from the string
                xmlfh.write("       Value=\"%s\" \n" % ( enumvaluestring ) )
                xmlfh.write("       Count=\"%i\" " % ( string.count(enumvaluestring,"|") ) )
                xmlfh.write("/>\n")
            # After we are done with the enumerated values
            # close the tag
            xmlfh.write("    </DATA_ITEM>\n")
            # back up one row, so we don't skip one by mistake
            #
            if (genny_datatype == "Enumerated") or (genny_datatype == "EnumeratedWithDefault"):
                if i != 0 :
                    i -= 1
                    row = container_rows[i]
            # end of the data item while
        i += 1
    xmlfh.write("  </CONTAINER>\n")
    return data_item_cnt,data_item_enum_cnt,data_item_enum_string_cnt
    
# *************************************************************
# If this is run as a script
#
if __name__ == '__main__':

    genny_util.display_license("Genny Tab-Delim-to-XML")

    if len(sys.argv )< 3:
        print "Usage: python genny.py [project name] [filename]"
        sys.exit()
        
    try:
        project_name = sys.argv[1]
        input_name = sys.argv[2]
        projdir = "./projects/" 
        outputfiledir         = projdir + project_name + "/output/"
        inputfiledir          = projdir + project_name + "/input/"

        infilename  = inputfiledir + input_name + ".txt"
        xmlfilename = inputfiledir + input_name + ".xml"
        logfilename          = outputfiledir + "logs/tabdelim-ldd-to-xml.log"

        processareaname   = project_name + "_ProcessArea"
        transaction_schema = project_name + "_TransactionSchema"
    except:
        print "Error! Usage: python genny.py [project name] [filename]"
        sys.exit()

    logfh = open(logfilename,"w") # open the logfile

    # parse the input file into a set of rows
    print "infilename is =%s=" % infilename
    LDD_rows = parse_LDD_file(infilename,logfh)
    print "Found %i Rows" % len(LDD_rows)
    log_LDD_rows(LDD_rows,logfh)

    # creating the Genny LDD xml file
    xmlfh = open(xmlfilename,"w")  # open the xmlfile
    xml_header(logfh,xmlfh)        # output the xml header text
    xml_body(logfh,xmlfh,LDD_rows,inputfiledir) # output the body of the xml file
    xmlfh.close()
    logfh.close()

    
