#!/usr/bin/env python2
#
# Genny - xml schema generation system
# Copyright (C) 2001-2004  James C. McDonald
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the 
#    Free Software Foundation, Inc.
#    59 Temple Place - Suite 330
#    Boston, MA  02111-1307, USA.
#
# created by:  Jim C. McDonald
#              McDonald Computer Corp.
#
# This application was created to fill a need in the
# MISMO Servicing Workgroup
#
#
#  Changelog
# 04/20/2001 - started development
# 04/23/2001 - basic functionality working, first complete attribute 
#              DTD generated
# 04/24/2001 - tweaks to eliminate some bad syntax being generated, 
#              will always generate a DTD with and without comments
# 05/07/2001 - added copyright and decided on license approach
# 06/22/2001 - updated input parse to use new xml style LDD
# 06/22/2001 - updated to handle new containers
# 06/22/2001 - modifications for the new row style
# 06/22/2001 - updated version to .021
# 06/22/2001 - updated the output file names
# 06/23/2001 - cleanup and fixes, now version .022
# 06/23/2001 - added the printing of the definitions for the enum 
#              values in the comments
# 06/23/2001 - added the printing of the container definitions 
#              in the comments
# 06/23/2001 - added the enum value definition to the LDD_row
# 06/26/2001 - changes/fixes from Jacksonville meeting
#              problems with the structure
#              added code to handle Booleans as enum Y/N
#              improved spacing of enum list for readability
#              added logic to handle the new ARM containers
#              updates/fixes to xml cardinality
# 07/05/2001 - fixes/changes for next mismo rev
# 07/09/2001 - version .024
# 07/18/2001 - version .025
# 07/23/2001 - version .026 fixed nasty bug in enum_string_value
#              added ability to handle ID,IDREF,Fixed Data Types
#              added a word wrap to the comments output
# 07/30/2001 - changed file names
# 08/06/2001 - version .027 changes for v2 of servicing draft
# 10/13/2001 - version .030 changes for v4 of the servicing draft
#              starting to add support for xsd schema output
#              setup genny_dtdgen,genny_sqlgen,genny_xsdgen as
#              python modules
# 11/01/2001 - changed to v2.1 RC1
# 11/29/2001 - changed to handle escaping/unescaping the invalid xml
#              characters that may be in the definition fields
# 01/04/2002 - changed to v2.1 RC2
# 01/24/2002 - changed to v2.1 RC3 and version .032
# 02/05/2002 - changed to v2.1 final
# 02/15/2002 - added the generation of ele files .033
# 02/15/2002 - modified for python class template generation 
# 03/27/2002 - changed .034
# 03/27/2002 - started work on cobol template generation
# 04/23/2002 - changed .040 significant changes
# 07/18/2002 - changes for v2.2 DRAFT
# 08/02/2002 - resumed work on xsd schema output
# 08/03/2002 - first time the xsd schema stuff is nearly complete
# 08/04/2002 - changed .050
# 08/05/2002 - improvements to xsd schema stuff
# 08/08/2002 - xsd schema stuff seems to be functional
# 10/02/2002 - cleaned up the dtdgen,sqlgen modules,
#              no longer use the LDD_rows only use the LDDE_rows
# 11/20/2002 - changed .052
# 12/04/2002 - changed .060 - run docbook automatically,HTML LDD
#              automatically, the tab delimited to xml automatically,
#              consolidated dir structure and take tha input name for
#              the LDD as a command line argument and build the 
#              filenames from there.
# 12/04/2002 - .061 - added a project struct to the input and output
#              directories and a way to create a new project structure,
#              and a way to remove all of the output files out of
#              a structure
# 12/05/2002 - .062 - more improvements to the new project setup and
#              command line options
# 12/11/2002 - .063 - fixed parsing problem with EnumeratedWithDefault type
# 12/19/2002 - .064 - fixed schema annotation generation error
#                     the annotations for the enumerations needed to be 
#                     nested inside the enumeration declarations themselves and
#                     the annotation for the xmlns attribute was being included
#                     even though the xmlns attribute was being skipped
# 02/06/2003 - changed to new version numbering 0.1.0
#              * added new approved acronyms
#              * fixed filename errors that were a problem on non-windows
#                platforms
# 02/12/2003 - fixed LDD to HTML error with hard coding of Title
#              made extensions file commented out by default in DTD
#              generation v0.1.1
# 02/28/2003 - fixed dtdgen module and problem with 1 or more cardinality
# 03/06/2003 - modified output filenames for the apparently new filename
#              convention { dashes:bad, underscores: good ;-) }
# 05/05/2003 - version v0.1.2
# 07/25/2003 - version v0.1.3 enhancements to handle data item short name
# 07/31/2003 - added the generation of xslt maps
# 02/12/2004 - version v0.1.4 
# 02/12/2004 - corrected some of the horribly wrong hard coded years
#              from 2003 to 2004 :-)
# 04/07/2004 - version v0.1.5 
# 04/07/2004 - changed the --new option to handle the case where the directories
#              to be added exist and to just skip the error when they can't be
#              created
# 04/07/2004 - version v0.1.6 
# 04/16/2004 - based on feedback from Craig Foote the xml_escape routines 
#              were modified to be able to handle the assortment of 
#              non-ascii characters found in the MISMO repository, also needed 
#              to explicitly declare the genny_util module to be encoded 
#              as "iso-8859-1"
# 04/16/2004 - version v0.1.7 
# 05/19/2004 - version v0.1.8 
#              fixed the bug with the valid value definition in the DTDs
# 07/19/2004 - version v0.1.9 
# 07/26/2004 - changed the name of the "type" variables to unshadow builtins
#              modified the tab-delimited parse routines to handle data item
#              names on each of the enumerated value rows
#
#

import sys
import time
import string
import copy
import os

sys.path.append("./imports/")

#import genny_parseldd
import genny_parselddE
import genny_sqlgen
import genny_xsdgen
import genny_dtdgen
import genny_util
import genny_py_template
import genny_cobol_template
import genny_xslt_name_map

import mismoextensions

__version__ = "0.1.9"

# global for debugging level 
debug = 1

## Original LDD parse 
##def parse_LDD_file(filename,logfh):
##    """ **************************************************************
##     parses the Genny LDD xml file,
##     this is the original format and should be deprecated
##
##    # **************************************************************
##    # Parse the Genny LDD xml file and return a list of lists
##    #  i.e. the LDD_rows
##    #
##    # LDD_row map as of 06/23/2001
##    #   0   container id
##    #   1   container definition
##    #   2   container xmlname
##    #   3   data item id
##    #   4   data item xmlname
##    #   5   data item definition
##    #   6   data item xmldatatype
##    #   7   data item enum value
##    #   8   data item enum string value
##    #   9   data item enum value definition
##    """
##    print "Starting parse of LDD ..."
##    logfh.write("Starting parse of LDD ...\n")
##    LDD_rows = genny_parseldd.parseGennyLDD(filename)
##    print "Completed parse of LDD ..."
##    logfh.write("Completed parse of LDD ...\n")
##    return LDD_rows


def parse_LDDE_file(filename,logfh):
    """
    Parses the Genny LDD xml file, i.e. the Enhanced LDD format (04/23/2002)  
    """
    print "Starting parse of Enhanced LDD (%s) ...\n" %(filename)
    logfh.write("Starting parse of Enhanced LDD (%s) ...\n" %(filename))
    LDDE_rows = genny_parselddE.parseGennyLDDE(filename)
    print "Completed parse of Enhanced LDD ..."
    logfh.write("Completed parse of Enhanced LDD ...\n")
    return LDDE_rows


def log_LDD_rows(LDD_rows,logfh):
    """
     Output the LDD rows info to the log file for debugging 
    """
    row_count = 0
    for row in LDD_rows:
        # print row
        logfh.write("row# %4i: %-40s: %-60s: enum val: %s\n" % (row_count,row[0], row[3], row[7]))
        row_count += 1
        #if row_count == 5: break
    logfh.write("total number of data items processed = %i\n" % (row_count))

# #####################################################################

if __name__ == '__main__':
    """
    Executed if this is run as a script
    """    
    # output the program's license
    genny_util.display_license()

    # only continue if we have the proper number of arguments
    if len(sys.argv )< 3:
        ##print len(sys.argv)
        print "Not enough parameters!"
        print "Usage: python genny.py [project name] [filename or --new or --cleanup]"
        sys.exit()
        
    project_name = sys.argv[1]

    # if passed a project name and NEW, create a new project dir
    if sys.argv[2] == "--new":
        print ("Creating an empty new project for %s" % project_name)
        genny_util.make_new_project_dir(project_name)
        sys.exit()

    # if passed a project name and NEW, create a new project dir
    if sys.argv[2] == "--cleanup":
        print ("Cleaning up output dir for project %s" % project_name)
        genny_util.clean_up_project_dir(project_name)
        sys.exit()

    projdir = "./projects/" 

    logfilename           = projdir + project_name + "/output/logs/gennylog.log"
    outputfiledir         = projdir + project_name + "/output/"
    inputfiledir          = projdir + project_name + "/input/"

    sqlheaderfilename     = inputfiledir + "sqlbpheader.txt"
    xsd_headerfilename    = inputfiledir + "xsd_header.txt"
    xsd_structurefilename = inputfiledir + "xsd_structure.txt"

    # for class template generation
    templatefilename           = outputfiledir + "templates/python/template.txt"
    classheaderfilename        = inputfiledir + "class_bpheader.py"
    classstructurefilename     = inputfiledir + "class_structure.py"

    # for cobol template generation
    coboltemplatefilename      = outputfiledir + "templates/cobol/template.txt"
    cobolheaderfilename        = inputfiledir + "cobol_bpheader.txt"
    cobolstructurefilename     = inputfiledir + "cobol_structure.txt"
    # build the input/output filenames
    try:
        input_name = sys.argv[2] 
        # for DTD generation
        dtdfilenames = []
        # two more names for the short names with and without comments
        dtdfilename = outputfiledir + input_name + "_ShortNames_With_Comments.dtd"
        dtdfilenames.append(dtdfilename)
        dtdfilename = outputfiledir + input_name + "_ShortNames.dtd"
        dtdfilenames.append(dtdfilename)
        # standard names with and without comments
        dtdfilename = outputfiledir + input_name + "_With_Comments.dtd"
        dtdfilenames.append(dtdfilename)
        dtdfilename = outputfiledir + input_name + ".dtd"
        dtdfilenames.append(dtdfilename)
        #
        dtd_ext_filename = outputfiledir +  input_name + "_Extensions.dtd"
        infilenameE = inputfiledir + input_name + ".xml"
        # for SQL DDL generation
        sqlfilename = outputfiledir + input_name + ".sql"
        # for xsd schema generation
        xsdfilenames = []
        xsdfilename = outputfiledir + input_name + "_With_Comments.xsd"
        xsdfilenames.append(xsdfilename)
        xsdfilename = outputfiledir + input_name + ".xsd"
        xsdfilenames.append(xsdfilename)
        # two more for the short names with and without comments
        xsdfilename = outputfiledir + input_name + "_ShortNames_With_Comments.xsd"
        xsdfilenames.append(xsdfilename)
        xsdfilename = outputfiledir + input_name + "_ShortNames.xsd"
        xsdfilenames.append(xsdfilename)
        xsd_infilename = infilenameE
    except:
        print "Error: You must pass the name of the input LDD to this script."
        print "This file is assumed to be in the ./input directory."
        sys.exit()

    logfh = open(logfilename,"w") # open the logfile
    
    # convert the Tab Delimited format to the XML format
    print "Converting tab delimited format to XML"
    command = "python TabDelim-to-GennyLDD.py" + " " + project_name + " " + input_name
    os.system(command)

    # parse the xml LDD into a python list of lists
    # the LDD_rows
    LDDE_rows = parse_LDDE_file(infilenameE,logfh) 

    # log the rows for later analysis if we are
    # really debugging, otherwise we don't really need to log the rows
    if debug > 0 : log_LDD_rows(LDDE_rows,logfh)

    # create the DTD's
    print "Generating the DTDs for %s " % input_name
    dtd_types_to_create = [["YES","YES"],["NO","YES"],["YES","NO"],["NO","NO"]]
    k = 0
    for dtd_type in dtd_types_to_create :    
        dtdgen = genny_dtdgen.dtd_generator(
                                           genny_version=__version__,
                                           debug=debug,
                                           headerfilename=dtdfilenames[k],
                                           logfh=logfh,
                                           dtd_ext_filename=dtd_ext_filename,
                                           LDD_rows=LDDE_rows,
                                           includecomments=dtd_type[0],
                                           input_name=input_name,
                                           project_name=project_name,
                                           use_short_name=dtd_type[1]
                                           )
        dtdgen.run()
        k += 1
                                 
    # create the SQL DDL
    print "Generating the SQL DDL"
    genny_sqlgen.sql_timetest()
    sqlfh = open(sqlfilename,"w") # open the SQL DDL file
    genny_sqlgen.sql_header(logfh,sqlfh,__version__,sqlheaderfilename)       # output the SQL header text
    # 10/02/02 making a copy of the rows should not be needed,
    # check what the sqlgen is doing to corrupt the rows for subsequent use
    copy_LDDE_rows = copy.deepcopy(LDDE_rows)
    genny_sqlgen.sql_tables(logfh,sqlfh,copy_LDDE_rows,"NO",debug)
    sqlfh.close()

    # create the xsd schema files
    print "Generating the xsd Schema"
    xsd_types_to_create = [["YES","NO"],["NO","NO"],["YES","YES"],["NO","YES"]]
    k = 0
    for xsd_type in xsd_types_to_create :
        xsdfh = open(xsdfilenames[k],"w") # open the w3C schema file
        genny_xsdgen.xsd_header(logfh,xsdfh,debug,__version__,xsd_headerfilename) # output the header text
        genny_xsdgen.xsd_structure(logfh,xsdfh,debug,xsd_structurefilename)       # output the overall structure text
        genny_xsdgen.xsd_containers(logfh,xsdfh,LDDE_rows,xsd_type[0],debug,xsd_type[1])  # output the containers in attribute style
        xsdfh.write("\n</xsd:schema>")  
        xsdfh.close()
        k += 1

##  xsdfh = open(xsdfilename,"w") # open the w3C schema file
##  genny_xsdgen.xsd_header(logfh,xsdfh,debug,__version__,xsd_headerfilename) # output the header text
##  genny_xsdgen.xsd_structure(logfh,xsdfh,debug,xsd_structurefilename)       # output the overall structure text
##  genny_xsdgen.xsd_containers(logfh,xsdfh,LDDE_rows,"NO",debug)             # output the containers in attribute style
### genny_xsdgen.xsd_process_attr_groups(logfh,xsdfh,LDDE_rows,"NO",debug)
### genny_xsdgen.xsd_write_enum_datatypes(logfh,xsdfh,LDDE_rows,"NO",debug)
##  xsdfh.write("\n</xsd:schema>")  
##  xsdfh.close()

    # create the templatefile
    print "Generating the python template(s)"
    templatefh = open(templatefilename,"w") # open the templatefile
    genny_py_template.template_header(logfh,templatefh,debug,__version__,classheaderfilename)
    genny_py_template.template_structure(logfh,templatefh,debug,classstructurefilename)
    genny_py_template.template_containers_attr(logfh,templatefh,LDDE_rows,type,debug,project_name) 
    templatefh.close()  

    # create the templatefile
    print "Generating the cobol tempate(s)"
    templatefh = open(coboltemplatefilename,"w") # open the templatefile
    genny_cobol_template.template_header(logfh,templatefh,debug,__version__,cobolheaderfilename)
    genny_cobol_template.template_structure(logfh,templatefh,debug,cobolstructurefilename)
    genny_cobol_template.template_containers_attr(logfh,templatefh,LDDE_rows,type,debug,project_name) 
    templatefh.close()  
    

    # create the XSLT maps
    xslt_map_gen = genny_xslt_name_map.xslt_name_map_generator(
                                                  genny_version=__version__,
                                                  debug=debug,
                                                  logfh=logfh,
                                                  LDD_rows=LDDE_rows,
                                                  project_name=project_name,
                                                  project_dir=projdir
                                                  )
    xslt_map_gen.run()

    logfh.close() # close the log file

    # create the HTML version of the LDD
    print "Generating the HTML LDD"
    command = "python GennyLDD-to-html.py" + " " + project_name + " " + input_name
    os.system(command)
    
    # create the docbook chapter3
    print "Generating the Docbook Chapter 3 XML"
    command = "python GennyLDD-to-docbook-chapter3.py" + " " + project_name + " " + input_name
    os.system(command)

    print "Genny All Done ===="
    sys.exit(0) 
# **************************************************************    
