#
# Genny - xml schema generation system
# Copyright (C) 2001-2002  James C. McDonald
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the 
#    Free Software Foundation, Inc.
#    59 Temple Place - Suite 330
#    Boston, MA  02111-1307, USA.
#
# created by:  Jim C. McDonald
#              McDonald Computer Corp.
#
# This application was created to fill a need in the
# MISMO Servicing Workgroup
#

# 06/23/2001 - added data item enum value definition to output
# 04/23/2002 - started the process of supporting the enhanced data
#              format
#

# LDD_row map as of 07/25/2003
#   0   container id
#   1   container definition
#   2   container xmlname
#   3   data item id
#   4   data item xmlname
#   5   data item definition
#   6   data item xmldatatype
#   7   data item enum value
#   8   data item enum string value
#   9   data item enum value definition
#  10   idref reference
#  11   fixed value 
#  ** rest added after 04/23/2002 ******
#  12   container_xmlsequence
#  13   container_primarykeyitem 
#  14   container_implementationnote
#  15   container_example
#  16   container_usedbyprocessarea
#  17   container_docgraphicfilename
#  18   container_defaultminoccurs
#  19   container_defaultmaxoccurs
#  20   container_parentcontainerlist
#  21   container_childrencontainerlist
#  22   parent_container_list (discrete items)
#  23   child_container_list (discrete items)
#  24   data_item_xmlsequence
#  25   data_item_xmlminoccurs
#  26   data_item_xmlmaxoccurs
#  27   data_item_implementationnote
#  28   data_item_example
#  29   data_item_usedbyprocessarea
#  30   data_item_datamappingnote
#  31   data_item_sourcedocument
#  32   data_item_minlength
#  33   data_item_maxlength
#  34   data_item_regexpattern
#  35   data_item_retainwhitespaceindicator
#  36   data_item_numericprecision
#  37   data_item_numericscale
#  38   data_item_mininclusive
#  39   data_item_maxinclusive
#  ** added 07/25/2003
#  40   data_item_xmlshortname

import xml.dom.minidom 
import sys
import time
import string

def getText(nodelist):
	rc = ""
	for node in nodelist:
		if node.nodeType == node.TEXT_NODE:
			rc = rc + node.data
	return rc

def hasChildByName(nodelist,inname):
	hit = 0
	for node in nodelist:
		if node.nodeName == inname:
			hit = 1
	return hit

# **************************************************************
# parse the list of child container/min/max occurs into
# discrete components
#
def parse_childlist(logfh,inputstring):
	childlist = []
	child_rows = string.split(inputstring,",")
	for row in child_rows:
		new_row = []
		items = string.split(row,":")
		for item in items:
			new_row.append(item)
		childlist.append(new_row)
	return childlist

# **************************************************************
# Primary Routine of this module
#
def parseGennyLDDE(inputfilename):
	data_item_enum_cnt = 0
	data_item_enum_string_cnt = 0
	data_item_cnt = 0
	row_cnt = 0

	LDD_rows = []
	row = []

	print "\nStarting GennyLDD Parse ==="
	dom1 = xml.dom.minidom.parse(inputfilename)
	ldd = dom1.getElementsByTagName("GENNY_LDD")

	# retrieve the attributes of the transaction_schema element
	transaction_schema_nodes = dom1.getElementsByTagName("TRANSACTION_SCHEMA")
	transaction_schema_id = transaction_schema_nodes[0].getAttribute("Id")
	transaction_schema_processarea = transaction_schema_nodes[0].getAttribute("ProcessArea")

	for node in transaction_schema_nodes[0].childNodes:
		if node.nodeName == "CONTAINER":
			# retrieve the attributes of the container element
			container_id                    = node.getAttribute("Id")
			container_xmlname               = node.getAttribute("XMLName")
			container_definition            = node.getAttribute("Definition")
			container_xmlsequence           = node.getAttribute("XMLSequence")
			container_primarykeyitem        = node.getAttribute("PrimaryKeyItem") 
			container_implementationnote    = node.getAttribute("ImplementationNote")
			container_example               = node.getAttribute("Example")
			container_usedbyprocessarea     = node.getAttribute("UsedByProcessArea")
			container_docgraphicfilename    = node.getAttribute("DocGraphicFilename")
			container_defaultminoccurs      = node.getAttribute("DefaultMinOccurs")
			container_defaultmaxoccurs      = node.getAttribute("DefaultMaxOccurs")
			container_parentcontainerlist   = node.getAttribute("ParentContainerList")
			container_childrencontainerlist = node.getAttribute("ChildrenContainerList")

			# retrieve the list of parent(s)
			parent_container_list = []
			if hasChildByName(node.childNodes,"PARENT_CONTAINER"):
				for xnode in node.childNodes:
					if xnode.nodeName == "PARENT_CONTAINER":
						parent_container_id = xnode.getAttribute("Id")
						parent_container_list.append(parent_container_id) 

			# retrieve the list of children 
			child_container_list = []
			if hasChildByName(node.childNodes,"CHILD_CONTAINER"):
				for xnode in node.childNodes:
					if xnode.nodeName == "CHILD_CONTAINER":
						child_container_id           = xnode.getAttribute("Id")
						child_container_xmlminoccurs = xnode.getAttribute("XMLMinOccurs")
						child_container_xmlmaxoccurs = xnode.getAttribute("XMLMaxOccurs")
						child_container_row = []
						child_container_row.append(child_container_id)
						child_container_row.append(child_container_xmlminoccurs)
						child_container_row.append(child_container_xmlmaxoccurs)
						child_container_list.append(child_container_row)

			for node in node.childNodes:
				if node.nodeName == "DATA_ITEM":
					data_item_cnt += 1
					# retrieve the attributes of the data_item element
					data_item_id                 = node.getAttribute("Id")
					data_item_xmlname            = node.getAttribute("XMLName")
					data_item_xmldatatype        = node.getAttribute("XMLDataType")
					data_item_definition         = node.getAttribute("Definition")
					data_item_xmlsequence        = node.getAttribute("XMLSequence")
					data_item_xmlminoccurs       = node.getAttribute("XMLMinOccurs")
					data_item_xmlmaxoccurs       = node.getAttribute("XMLMaxOccurs")
					data_item_implementationnote = node.getAttribute("ImplementationNote")
					data_item_example            = node.getAttribute("Example")
					data_item_usedbyprocessarea  = node.getAttribute("UsedByProcessArea")
					data_item_datamappingnote    = node.getAttribute("DataMappingNote")
					data_item_sourcedocument     = node.getAttribute("SourceDocument")
					data_item_idrefref           = node.getAttribute("IDREFReference")
					data_item_fixedval           = node.getAttribute("FixedValue")
					data_item_minlength          = node.getAttribute("minLength")
					data_item_maxlength          = node.getAttribute("maxLength")
					data_item_regexpattern       = node.getAttribute("regexPattern")
					data_item_retainwhitespaceindicator = node.getAttribute("RetainWhitespaceIndicator")
					data_item_numericprecision   = node.getAttribute("numericPrecision") 
					data_item_numericscale       = node.getAttribute("numericScale")
					data_item_mininclusive       = node.getAttribute("minInclusive")
					data_item_maxinclusive       = node.getAttribute("maxInclusive")
					data_item_xmlshortname       = node.getAttribute("XMLShortName")

					# if the current node does not have any data_item_enum_value elements
					# below it then we can add the row to the list
					if not hasChildByName(node.childNodes,"DATA_ITEM_ENUM_VALUE"):
						row.append(container_id)
						row.append(container_definition)
						row.append(container_xmlname)
						row.append(data_item_id)
						row.append(data_item_xmlname)
						row.append(data_item_definition)
						row.append(data_item_xmldatatype)
						data_item_enum_value = ""
						row.append(data_item_enum_value)
						data_item_enum_string_value = ""
						row.append(data_item_enum_string_value)
						data_item_enum_def = ""
						row.append(data_item_enum_def)
						row.append(data_item_idrefref)
						row.append(data_item_fixedval)
						# ADD new rows here for Enhanced
						row.append(container_xmlsequence)
						row.append(container_primarykeyitem)
						row.append(container_implementationnote)
						row.append(container_example)
						row.append(container_usedbyprocessarea)
						row.append(container_docgraphicfilename)
						row.append(container_defaultminoccurs)
						row.append(container_defaultmaxoccurs)
						row.append(container_parentcontainerlist)
						row.append(container_childrencontainerlist)
						row.append(parent_container_list)
						row.append(child_container_list)
						row.append(data_item_xmlsequence)
						row.append(data_item_xmlminoccurs)
						row.append(data_item_xmlmaxoccurs)
						row.append(data_item_implementationnote)
						row.append(data_item_example)
						row.append(data_item_usedbyprocessarea)
						row.append(data_item_datamappingnote)
						row.append(data_item_sourcedocument)
						row.append(data_item_minlength)
						row.append(data_item_maxlength)
						row.append(data_item_regexpattern)
						row.append(data_item_retainwhitespaceindicator)
						row.append(data_item_numericprecision)
						row.append(data_item_numericscale)
						row.append(data_item_mininclusive)
						row.append(data_item_maxinclusive)
						row.append(data_item_xmlshortname)
						
						LDD_rows.append(row)
						row_cnt += 1
						row = []
						#
					else :
						for node in node.childNodes:
							if node.nodeName == "DATA_ITEM_ENUM_VALUE":
								data_item_enum_cnt += 1 
								data_item_enum_value = node.getAttribute("Value")
								data_item_enum_def = node.getAttribute("Definition")

								row.append(container_id)
								row.append(container_definition)
								row.append(container_xmlname)
								row.append(data_item_id)
								row.append(data_item_xmlname)
								row.append(data_item_definition)
								row.append(data_item_xmldatatype)
								row.append(data_item_enum_value)
								data_item_enum_string_value = ""
								row.append(data_item_enum_string_value)
								row.append(data_item_enum_def)
								# added 12-12-02
								row.append(data_item_idrefref)
								row.append(data_item_fixedval)
								# ADD new rows here for Enhanced
								row.append(container_xmlsequence)
								row.append(container_primarykeyitem)
								row.append(container_implementationnote)
								row.append(container_example)
								row.append(container_usedbyprocessarea)
								row.append(container_docgraphicfilename)
								row.append(container_defaultminoccurs)
								row.append(container_defaultmaxoccurs)
								row.append(container_parentcontainerlist)
								row.append(container_childrencontainerlist)
								row.append(parent_container_list)
								row.append(child_container_list)
								row.append(data_item_xmlsequence)
								row.append(data_item_xmlminoccurs)
								row.append(data_item_xmlmaxoccurs)
								row.append(data_item_implementationnote)
								row.append(data_item_example)
								row.append(data_item_usedbyprocessarea)
								row.append(data_item_datamappingnote)
								row.append(data_item_sourcedocument)
								row.append(data_item_minlength)
								row.append(data_item_maxlength)
								row.append(data_item_regexpattern)
								row.append(data_item_retainwhitespaceindicator)
								row.append(data_item_numericprecision)
								row.append(data_item_numericscale)
								row.append(data_item_mininclusive)
								row.append(data_item_maxinclusive)
								row.append(data_item_xmlshortname)
						
								LDD_rows.append(row)
								row.append(data_item_idrefref)
								row.append(data_item_fixedval)
								row_cnt += 1
								row = []
								
								data_item_enum_value = ""
								data_item_enum_def = ""
								#
							elif node.nodeName == "DATA_ITEM_ENUM_DTD_STRING":
								data_item_enum_string_cnt += 1 
								data_item_enum_string_value = node.getAttribute("Value")

								row.append(container_id)
								row.append(container_definition)
								row.append(container_xmlname)
								row.append(data_item_id)
								row.append(data_item_xmlname)
								row.append(data_item_definition)
								row.append(data_item_xmldatatype)
								row.append(data_item_enum_value)
								row.append(data_item_enum_string_value)
								row.append(data_item_enum_def)
								row.append(data_item_idrefref)
								# ADD new rows here for Enhanced
								row.append(container_xmlsequence)
								row.append(container_primarykeyitem)
								row.append(container_implementationnote)
								row.append(container_example)
								row.append(container_usedbyprocessarea)
								row.append(container_docgraphicfilename)
								row.append(container_defaultminoccurs)
								row.append(container_defaultmaxoccurs)
								row.append(container_parentcontainerlist)
								row.append(container_childrencontainerlist)
								row.append(parent_container_list)
								row.append(child_container_list)
								row.append(data_item_xmlsequence)
								row.append(data_item_xmlminoccurs)
								row.append(data_item_xmlmaxoccurs)
								row.append(data_item_implementationnote)
								row.append(data_item_example)
								row.append(data_item_usedbyprocessarea)
								row.append(data_item_datamappingnote)
								row.append(data_item_sourcedocument)
								row.append(data_item_minlength)
								row.append(data_item_maxlength)
								row.append(data_item_regexpattern)
								row.append(data_item_retainwhitespaceindicator)
								row.append(data_item_numericprecision)
								row.append(data_item_numericscale)
								row.append(data_item_mininclusive)
								row.append(data_item_maxinclusive)
								row.append(data_item_xmlshortname)

								LDD_rows.append(row)
								row.append(data_item_fixedval)
								row_cnt += 1
								row = []
								
								data_item_enum_value = ""
								data_item_enum_def = ""
								data_item_enum_string_value
								#

	dom1.unlink() # release the minidom resources used
	print ""
	print "%5i data items processed." % (data_item_cnt)
	print "%5i data item valid values processed." % (data_item_enum_cnt)
	print "%5i enum strings created." % (data_item_enum_string_cnt)
	print "%5i total items processed." % (data_item_cnt+data_item_enum_cnt)
	print "%5i rows created." % (row_cnt) 
	print ""
	print "GennyLDD Parse Completed ==="
	return LDD_rows


