# Genny - xml schema generation system
# Copyright (C) 2001-2002  James C. McDonald
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the 
#    Free Software Foundation, Inc.
#    59 Temple Place - Suite 330
#    Boston, MA  02111-1307, USA.
#
# created by:  Jim C. McDonald
#              McDonald Computer Corp.
#
# This application was created to fill a need in the
# MISMO Servicing Workgroup

# 10/18/2001 to handle EXTENSION/EXTENSION_SECTION
# 10/02/2002 cleanups and improvements to used the enhanced LDD
#            meta data

import sys
import time
import string
import genny_util

generic_sql_datatype = "VARCHAR2(255)"

# for SQL DDL generation

def sql_timetest():
    """ **************************************************************
    Output the current time
    """
    currtime = 0.00
    currtime = time.time()
    ##print "current time is %f" % currtime


def sql_header(logfh,sqlfh,__version__,sqlheaderfilename):
    """ **************************************************************
    Output the header to the SQL DDL file
    """
    print "Adding header text to sql ..."
    currtime = time.time() # get the current time 
    sqlfh.write("/*\n *  Generated by Genny v%s on %s\n */\n" % (__version__,(time.ctime(currtime))) )
    sqlbpfh = open(sqlheaderfilename,"r")
    sqlbptext = sqlbpfh.readlines()
    sqlfh.writelines(sqlbptext)
    logfh.write("Added boilerplate text to sql...\n")
    sqlbpfh.close()


def sql_tables(logfh,sqlfh,LDD_rows,commentflag,debug):
    """ **************************************************************
    Output the SQL DDL for the containers in the input LDD to the
    SQL DDL output file
    """
    print "Adding containers as SQL tables ..."
    row = LDD_rows[0] # extract the first row
    prev_container_name = row[2] # prime the prev name logic
    container_count = 0          # prime the container count
    containers = []
    container_rows = []
    i = 0
    while i < len(LDD_rows):
        row = LDD_rows[i]
        if prev_container_name == row[2] :
            container_rows.append(row)
            logfh.write("   appending subelement as table attribute %s\n" % (row[4]) )
        else :
            # handle end of a container
            print "  Processing %s container as table" % prev_container_name
            logfh.write("Processing %s container as table\n" % (prev_container_name) )
            containers.append(prev_container_name) # store the container name
            prev_container_name = row[2] # reset the prev container name
            container_count += 1
            #
            sql_write_table(logfh,sqlfh,container_rows,commentflag,debug)
            #
            container_rows = [] # reset the
            container_rows.append(row)
            logfh.write("   appending subelement %s\n" % (row[4]) )
        i += 1  # index to the next row in the LDD
                # if we've reached the end handle the last container
        if i == len(LDD_rows):
            sys.stderr.write( "  Processing %s container as table\n" % prev_container_name )
            logfh.write("Processing %s container as table\n" % (prev_container_name) )
            container_count += 1
            sql_write_table(logfh,sqlfh,container_rows,commentflag,debug)
    print "%i containers have been processed.\n" % (container_count)
    logfh.write("%i containers have been processed.\n" % (container_count))

def sql_write_table(logfh,sqlfh,container_rows,commentflag,debug):
    """ **************************************************************
    write the SQL DDL for a single container LDD to the SQL DDL
    output file
    10/02/02 - revised for the enhanced LDD info
    """
    row = container_rows[0]
    sqlfh.write("\n\n")
    sqlfh.write("/*  =================================================================  */\n")
    sqlfh.write("/*   %-40s                          */\n" % (row[2]))
    sqlfh.write("/*  =================================================================  */\n")
    #

    if row[2] == "DEFAULT":
        sqlfh.write("create table %s (\n" % ("DEFAULT_TABLE"))
    else:
        sqlfh.write("create table %s (\n" % (row[2]))

    #--------------------------------------------
    # write the primary key column
    # code added july 09, 2001 for new key handling
    if len(row[2]) !=0: # write the primary key column
        sqlfh.write("%-50s %s NOT NULL " % ((row[2]+"ID"),generic_sql_datatype) )
    #--------------------------------------------
    # add the appropriate columns for the foreign keys for each table
    # associated with a parent or child container
    #
    # handle the parent

    if len(row[22]) == 0:
        # no parents or childred
        pass
    elif row[22][0] == 'Root':
        pass
    else:
        for par_c_list in row[22]:
            sqlfh.write(",\n")
            sqlfh.write("%-50s %s " % (string.upper(par_c_list+"ID"),generic_sql_datatype))
    # then handle the children
    if len(row[23]) == 0:
        # no children
        pass
    elif row[23][0][0] == 'ANY':
        # ANY content
        pass
    else:
        for child_c_list in row[23]:
            sqlfh.write(",\n")
            sqlfh.write("%-50s %s " % (string.upper(child_c_list[0]+"ID"),generic_sql_datatype))
    #--------------------------------------------
    # ********************************************
    i = 0
    row_id_hit = 0

    sys.stderr.write("Container %s length %i!\n" % (row[2],len(container_rows)))
    while i < len(container_rows):
        c_row = container_rows[i]
        if c_row[2] == "DEFAULT":
            c_row[2] = "DEFAULT_TABLE"

        if row_id_hit == 1:
            row_id_hit = 0
        else:
            if len(container_rows) != 1:
                sqlfh.write(",\n") 

        if c_row[6] == "Enumerated":
            # we don't need the enum string, but we do need to know
            # how many rows to skip ahead and j gives us that 
            enum_string_value,j,enum_rows = genny_util.get_enum_string_value(i,container_rows,debug)
            sqlfh.write("%-50s %s" % (c_row[4],generic_sql_datatype) )
        else:
            if c_row[6] == "ID": #skip the ID or it will be in the table twice
                row_id_hit = 1
            else:
                sqlfh.write("%-50s %s" % (c_row[4],generic_sql_datatype) )

        # skip to the next row to be processed  
        if c_row[6] == "Enumerated" and j != 0:
            # skip the enumerated value rows that are passed in 
            i=i+j
        else:
            i += 1

    sqlfh.write("\n);\n\n") # close the parenthesis for the create table
    #--------------------------------------------
    # the table should be complete now
    #--------------------------------------------
    # write the unique index and primary key stuff
    sqlfh.write("/*\n  Primary Key and Unique Index declaration \n */\n")
    sqlfh.write("CREATE UNIQUE INDEX XPK%s ON %s (%s ASC);\n" % (row[2],row[2],(row[2]+"ID")))
    sqlfh.write("ALTER TABLE %s ADD (PRIMARY KEY (%s) );\n" % (row[2],(row[2]+"ID")))
    #--------------------------------------------
    # write the foreign key declarations

    # handle the parent
##    if len(row[22]) == 0:
##        # no parents or children
##        pass
##    elif row[22][0] == 'Root':
##        pass
##    else:
##        if len(row[22]) > 0: sqlfh.write("/*\n  FKs for Parent Container(s) \n */\n")
##        for par_c_list in row[22]:
##            sqlfh.write("ALTER TABLE %s ADD  (  FOREIGN KEY  (%s) REFERENCES %s ) ;\n" % \
##                        (row[2],string.upper((par_c_list+"ID")),par_c_list) )

    # then handle the children
    if len(row[23]) == 0:
        # no children
        pass
    elif row[23][0][0] == 'ANY':
        # ANY content
        pass
    else:
        if len(row[23]) > 0: sqlfh.write("/*\n  FKs for Child Container(s) \n */\n")
        for child_c_list in row[23]:
            if child_c_list[0] == 'DEFAULT': child_c_list[0] = 'DEFAULT_TABLE'
            sqlfh.write("ALTER TABLE %s ADD  (  FOREIGN KEY  (%s) REFERENCES %s ) ;\n" % \
                        (row[2],string.upper((child_c_list[0]+"ID")),child_c_list[0]) )
