# -*- coding: iso-8859-1 -*-
# Genny - XML schema generation system
# Copyright (C) 2001-2003  James C. McDonald
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the 
#    Free Software Foundation, Inc.
#    59 Temple Place - Suite 330
#    Boston, MA  02111-1307, USA.
#
# created by:  Jim C. McDonald
#              McDonald Computer Corp.
#
# This application was created to fill a need in the
# MISMO Servicing Workgroup

# 12-04-2002 wrote the code to create a new project directory
# 12-04-2002 wrote routine to remove all the files from an output dir
# 12-05-2002 added default text when creating a new project and copied
#            the dtd to the input directory
# 02-06-2003 added the license display to the util module
# 07-25-2003 updated to GennyLDD v2.02
# 07-25-2003 added routine (getShortName)to determine the proper short
#            name for a given data item when passed a parsed LDD row object
# 10-24-2003 handle short names that are provided as "Dxxxx" vs. an integer
# 10-29-2003 changed name used if no short name is provided
# 04-07-2004 ignore errors in creating new directorites and continue and try
#            to create the template files
# 04-16-2004 modified the xml_escape routines to handle the non-ascii characters
#            in the MISMO repository and created an xml_escape_lite() routine
#            that just does the proper subset of characters really needed
# 07-26-2004 renamed str variables to wrk_string to unshadow str builtin
#

import sys
import time
import string
import os
import shutil

# boilerplate text for creating a new project from scratch
#
boilerplate_text_list =[
"""
""",
"""

/*
 *
 * Copyright 2004 Mortgage Industry Standards Maintenance Organization (MISMO).      
 * All rights reserved.                                                              
 *                                                                                   
 * Permission to use, copy, modify, and distribute the MISMO                         
 * sql and its accompanying documentation for any purpose and without                
 * fee is hereby granted in perpetuity, provided that the above copyright            
 * notice and this paragraph appear in all copies. The copyright holders make        
 * no representation about the suitability of the DTD for any purpose.               
 *                                                                                   
 *                                                                
 * /
""",
"""
<xsd:schema xmlns:xsd = "http://www.w3.org/2001/XMLSchema"
	    elementFormDefault   ="unqualified"
            attributeFormDefault ="unqualified"
>

<!-- ================================================================================= -->
<!--                                                                                   -->
<!-- xxx.xsd                                                                           -->
<!-- Version x.x                                                                       -->
<!-- Copyright 2004 Mortgage Industry Standards Maintenance Organization (MISMO).      -->
<!-- All rights reserved.                                                              -->
<!--                                                                                   -->
<!-- Permission to use, copy, modify, and distribute the MISMO                         -->
<!-- schema and its accompanying documentation for any purpose and without             -->
<!-- fee is hereby granted in perpetuity, provided that the above copyright            -->
<!-- notice and this paragraph appear in all copies. The copyright holders make        -->
<!-- no representation about the suitability of the DTD for any purpose.               -->
<!--                                                                                   -->
<!-- Schema for xxxxxxxxxxxxxxxxx transaction                                          -->
<!--                                                                                   -->
<!-- ================================================================================= -->
""",
"""
<!-- ================================================================================= -->
<!--                                                                                   -->
<!-- xxxxxxxxxxxxxxxxxxx                                                               -->
<!--                                                                                   -->
<!-- ================================================================================= -->
""",
"""
#
# Genny - xml schema generation system
# Copyright (C) 2004  James C. McDonald
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the 
#    Free Software Foundation, Inc.
#    59 Temple Place - Suite 330
#    Boston, MA  02111-1307, USA.
#
# created by:  Jim C. McDonald
#              McDonald Computer Corp.
#
# This application was created to fill a need in the
# MISMO Servicing Workgroup
#
#
""",
"""
import sys
import os
import time
import string

#
#
class servicing_transfer_obj(ldd_rows):

    def __init__(self):
        pass

#
#
""",
"""
      *
      * created by Genny - COBOL template code 
      * 
      * Copyright (C) 2004  James C. McDonald
      *
      * This program is free software; you can redistribute it and/or
      * modify it under the terms of the GNU General Public License
      * as published by the Free Software Foundation; either version 2
      * of the License, or (at your option) any later version.
      *
      * This program is distributed in the hope that it will be useful,
      * but WITHOUT ANY WARRANTY; without even the implied warranty of
      * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
      * GNU General Public License for more details.
      * 
      * You should have received a copy of the GNU General Public License
      * along with this program; if not, write to the 
      *    Free Software Foundation, Inc.
      *    59 Temple Place - Suite 330
      *    Boston, MA  02111-1307, USA.
      *
      * created by:  Jim C. McDonald
      *              McDonald Computer Corp.
      *
      * This application was created to fill a need in the
      * MISMO Servicing Workgroup.
""",
"""
      * 
       fd  mismo-loan-file
           label records are standard.
       01  mismo-loan-rcd.
"""
]

def clean_up_project_dir(project_name):
    """ Removes all of the files in the various
        output directories for a project.
    """    
    project_path = "./projects/" + project_name

    dir_list = [ "/output/",
                 "/output/logs/",
                 "/output/ele/",
                 "/output/templates/",
                 "/output/templates/python/",
                 "/output/templates/cobol/"
               ]
    try:
        for o_dir in dir_list:
            print ("Searching %s dir" %( project_path + o_dir))
            o_dir_list = os.listdir(project_path + o_dir)
##            for file in o_dir_list:
##                print ("found file %s" % file)
            for filename in o_dir_list:
##                print ("found file %s" % (project_path + o_dir + filename))
                if os.path.isfile(project_path + o_dir + filename):
                    print ("removing file: %s" % filename)
                    os.remove(project_path + o_dir + filename)
                    pass
                else:
                    print ("not removing dir: %s" % filename)
        return 0
    except (IOError,OSError):
        return 1


def make_new_proj_mkdir(pathname):
    """ Created to handle the case where the directories already
        exist but you still want the new template files to be 
        created.
    """
    if os.path.isdir(pathname):
        print "Directory %s already exists, skipping." % pathname
    else:
        os.mkdir(pathname)

def make_new_project_dir(project_name):
    """ Creates a template directory and file structure
        for a project.
    """
    project_path = "./projects/" + project_name
    try:
        make_new_proj_mkdir(project_path)
        make_new_proj_mkdir(project_path + "/input")
        make_new_proj_mkdir(project_path + "/input/docbook-components")
        make_new_proj_mkdir(project_path + "/output")
        make_new_proj_mkdir(project_path + "/output/logs")
        make_new_proj_mkdir(project_path + "/output/ele")
        make_new_proj_mkdir(project_path + "/output/templates")
        make_new_proj_mkdir(project_path + "/output/templates/python")
        make_new_proj_mkdir(project_path + "/output/templates/python/sax")
        make_new_proj_mkdir(project_path + "/output/templates/cobol")
        
        ##os.mkdir(project_path)
        ##os.mkdir(project_path + "/input")
        ##os.mkdir(project_path + "/input/docbook-components")
        ##os.mkdir(project_path + "/output")
        ##os.mkdir(project_path + "/output/logs")
        ##os.mkdir(project_path + "/output/ele")
        ##os.mkdir(project_path + "/output/templates")
        ##os.mkdir(project_path + "/output/templates/python")
        ##os.mkdir(project_path + "/output/templates/python/sax")
        ##os.mkdir(project_path + "/output/templates/cobol")

        # create default template files for the following
        # list of files in the input dir
        #
        # the first item in each list is the filename to
        # be created, and the second is the index into
        # the boilerplate text list for the actual text
        # to place into the file
        default_file_list = [ ["sqlbpheader.txt",1],
                              ["xsd_header.txt",2],
                              ["xsd_structure.txt",3],
                              ["class_bpheader.py",4],
                              ["class_structure.py",5],
                              ["cobol_bpheader.txt",6],
                              ["cobol_structure.txt",7],
                              ]
        for filenamelist in default_file_list:
            print "Creating %s" % filenamelist[0]
            fh = open( project_path + "/input/" + filenamelist[0], "w")
            fh.write(boilerplate_text_list[filenamelist[1]])
            fh.close()
        # copy all the files in the list to the input dir    
        files_to_copy_list = [
            ["./docs/dtd/gennyLDD-v202.dtd","gennyLDD-v202.dtd"]
            ]
        for filepathlist in files_to_copy_list:
            shutil.copyfile(filepathlist[0], project_path + "/input/"+filepathlist[1])
        #
        return 0
    except (IOError,OSError):
        return 1


def get_enum_string_value(i,container_rows,debug):
    """
     given a set of rows return the enum string, an count of rows
     to skip to jump past this enum value, and a list of the
     enum values and definitions
    """ 
    enum_string_value = ""
    enum_rows = []
    j = 0
    if debug > 1: sys.stderr.write("finding enum value for %s\n" % container_rows[i][4] )
    while i < len(container_rows):
        row = container_rows[i]
        if string.count(row[8],"|") != 0:
            enum_string_value = row[8]
            i += 1
            j += 1
            break
        else:
            # collect the enum values and definitions for the
            # comments 
            enum_row = []
            enum_row.append(row[7])
            enum_row.append(row[9])
            enum_rows.append(enum_row) 
        i +=1;
        j +=1;
    if enum_string_value[0:2] == ' |':
        enum_string_value = enum_string_value[3:]  # slice off the first sp|sp from the string      
    return enum_string_value,j,enum_rows

# *********************************************************************
#  created 07-25-2003
# modified 10-24-2003 - trim the 'D' off the short name if provided
# modified 10-29-2003 - changed name used if no short name is provided
#
def getShortName(row=[],count=0,debug=0):
    attribute_name = ''            # will be the short name 
    suggested_short_name = row[40] # short name from the LDD map
    if row[6] == 'ID':
        attribute_name = 'ID'      #All IDs have short names of ID
    elif row[6] == 'IDREF':
        attribute_name = row[4]    #IDREF type is unchanged
    elif row[6] == 'IDREFS':
        attribute_name = row[4]    #IDREFS type is unchanged
    else:
        if len(suggested_short_name) > 0:
            if debug >0: print "r%i c40 val:%s" % (count,str(row[40])),
            # TO FIX - replace with row[40] ROW40 SHORTNAME FIX
            #
            if suggested_short_name[0] == 'D':
                # trim the D off the suggested name (if provided)
                suggested_short_name = suggested_short_name[1:]
            attribute_name = ("D%04d" % int(suggested_short_name)) 
        else:
            # use the count if no name is provided
            attribute_name = ("D_ERROR_MISSING_SHORT_NAME_%04d" % count) 
    return attribute_name
                    
# routines to escape/unescape strings to be valid XML content
# added 11/29/2001 jcm
#
def xml_escape(wrk_string):
    wrk_string = string.replace(wrk_string, "&", "&amp;")
    wrk_string = string.replace(wrk_string, "<", "&lt;")
    wrk_string = string.replace(wrk_string, ">", "&gt;")
    wrk_string = string.replace(wrk_string, '"', "&quot;")
    wrk_string = string.replace(wrk_string, "'", "&apos;")
    wrk_string = string.replace(wrk_string, "`", "&apos;")
    wrk_string = string.replace(wrk_string,"","&apos;")
    wrk_string = string.replace(wrk_string,"","&quot;")
    wrk_string = string.replace(wrk_string,"","&quot;")
    wrk_string = string.replace(wrk_string,"","-")
   #
# Added Decimal 146 to eliminate apostrophe problem.
# added 04/16/2004 cwf
    wrk_string = string.replace(wrk_string, "", "&apos;")
# Added Decimal 147 to eliminate LEFT double quote problem.
# added 04/16/2004 cwf
    wrk_string = string.replace(wrk_string, "", "&quot;")
# Added Decimal 148 to eliminate RIGHT double quote problem.
# added 04/16/2004 cwf
    wrk_string = string.replace(wrk_string, "", "&quot;")
# Added Decimal 150 to eliminate a Hyphen problem.
# added 04/16/2004 cwf
    wrk_string = string.replace(wrk_string, "", "&apos;") 
   # 
   # report an character encoding error
    for char_in_string in wrk_string:
	ord_value = ord(char_in_string)
	if ord_value  > 127:
		print "ERROR: Found non ascii character with val %i in %s" % (ord_value,wrk_string)
		break
    #		   
    return(wrk_string)

# routines to escape/unescape strings to be valid XML content
# added 04/16/2004 jcm
#
def xml_escape_lite(wrk_string):
    wrk_string = string.replace(wrk_string, "&", "&amp;")
    wrk_string = string.replace(wrk_string, "<", "&lt;")
    wrk_string = string.replace(wrk_string, ">", "&gt;")
    wrk_string = string.replace(wrk_string, '"', "&quot;")
    wrk_string = string.replace(wrk_string, "'", "&apos;")
    return(wrk_string)

def xml_unescape(wrk_string):
    wrk_string = string.replace(wrk_string, "&amp;" , "&")
    wrk_string = string.replace(wrk_string, "&lt;"  , "<")
    wrk_string = string.replace(wrk_string, "&gt;"  ,">")
    wrk_string = string.replace(wrk_string, "&quot;",'"')
    wrk_string = string.replace(wrk_string, "&apos;","'")
    return(wrk_string)

def html_escape(wrk_string):
    wrk_string = string.replace(wrk_string, "&", "&amp;")
    wrk_string = string.replace(wrk_string, "<", "&lt;")
    wrk_string = string.replace(wrk_string, ">", "&gt;")
    wrk_string = string.replace(wrk_string, '"', "&quot;")
    wrk_string = string.replace(wrk_string, "'", "&#039;")
    return(wrk_string)

def display_license(progname="Genny"):
    """
    Display the license for Genny on the system console
    """
    print "======================================================================================="
    print ""
    print "%s : Copyright (c) 2001-2004 James C. McDonald" % progname
    print "Genny comes with ABSOLUTELY NO WARRANTY;"
    print "This is free software, and you are welcome to redistribute it under certain conditions; "
    print "Please see the included gpl-license.txt or http://www.gnu.org/copyleft/ for details."
    print ""
    print "======================================================================================="
