# Genny - xml schema generation system
# Copyright (C) 2001-2003  James C. McDonald
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the 
#    Free Software Foundation, Inc.
#    59 Temple Place - Suite 330
#    Boston, MA  02111-1307, USA.
#
# created by:  Jim C. McDonald
#              McDonald Computer Corp.
#

import sys
import time
import string
import genny_util
import TextFormatter
import mismoextensions
import operator

class xslt_name_map_generator:
    """ XSLT Name Map Generator

    Class to transform an LDD (as represented by LDD_rows) into XSLT maps from the
    long name to the short name and the reverse.
    
    This application was created to fill a need in the
    MISMO Servicing Workgroup.

    07/31/2003 created

    """
    
    def __init__(self,
                 genny_version='',
                 debug=0,
                 logfh='',
                 LDD_rows='',
                 project_name='default_proj_name',
                 project_dir='./projects/'
                 ):
        """
        Initialize the Name Map generator class.
        """
        self.genny_version=genny_version
        self.genny_xslt_name_map_gen_version='0.02'
        self.debug=debug
        self.logfh = logfh
        self.LDD_rows=LDD_rows
        self.project_name = project_name
        self.project_dir = project_dir
        # open the two files that will be created
        self.short_to_longfh = open(self.project_dir + self.project_name + '/output/'+self.project_name+'_short_to_long.xslt',"w") 
        self.long_to_shortfh = open(self.project_dir + self.project_name + '/output/'+self.project_name+'_long_to_short.xslt',"w")
        self.attribute_name_list1 = []
        self.attribute_name_list2 = []

    def cleanup(self):
        """
        Any closing housekeeping that is necessary.
        """
        self.long_to_shortfh.close()
        self.short_to_longfh.close()

    def run(self):
        """
        Perform all the steps necessary to generate the maps.  If the class has
        been initialized properly the XSLT Maps will be created when this method
        is executed.
        """
        # create the Short to Long Map
        self.maptypeflag = 'S2L'
        self.header(self.short_to_longfh,"Short To Long")
        self.containers_attr_style(self.LDD_rows,self.short_to_longfh)
        self.footer(self.short_to_longfh)
        # Create the Long to Short Map
        self.maptypeflag = 'L2S'
        self.header(self.long_to_shortfh,"Long To Short")
        self.containers_attr_style(self.LDD_rows,self.long_to_shortfh)
        self.footer(self.long_to_shortfh)
        # Create the HTML attribute Name Maps
        self.html_name_maps()
        # only call cleanup once
        self.cleanup()


    def header(self,headerfh,textname):
        """
        Places the boilerplate text in the xslt maps
        """
        print "Generating XSLT Map for: %s" % textname
        if self.debug > 0: print "Adding header std text to xslt map ..."
        toptext ='''<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">

<xsl:template match="/ | node() | @* | comment() | processing-instruction()">
  <xsl:copy>
    <xsl:apply-templates select="@* | node()"/>
  </xsl:copy>
</xsl:template>
'''
        currtime = time.time() # get the current time 
        headerfh.write("%s" % (toptext))
        headerfh.write("\n<!--Generated by Genny v%4s on %s -->\n\n" % (self.genny_version,(time.ctime(currtime))))

    def footer(self,headerfh):
        """
        Places the boilerplate text in the xslt maps
        """
        if self.debug > 0: print "Adding footer std text to xslt map ..."
        headerfh.write("\n\n</xsl:stylesheet>\n")


    def html_name_maps(self):
        outfh = open(self.project_dir + self.project_name + '/output/'+self.project_name+'_NameMap.html',"w")
        
        outfh.write("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0 Transitional//EN\">\n")
        currtime = time.time() # get the current time 
        outfh.write("<!--Generated by MISMO Genny XSLT Map v%s on %s -->\n" % (self.genny_xslt_name_map_gen_version,(time.ctime(currtime))))

        outfh.write("<html>\n" )
        outfh.write("<head>\n" )
        outfh.write("<meta http-equiv=\"Content-Type\" content=\"text/html; charset=ISO-8859-1\" />\n" )
        outfh.write("<title>MISMO %s Version 2.3.1 Data Item Standard/Short Name Map</title>\n" % self.project_name)
        outfh.write("</head>\n" )
        outfh.write("<body>\n" )

        outfh.write("<h1 align=\"center\">%s v2.3.1 Data Item Standard/Short Name Map</h1>\n" % self.project_name)

        outfh.write("<p>\n")
        outfh.write(
"""
Copyright 2004 Mortgage Bankers Association of America, Inc.<br /><br /> 

Permission to use, copy, modify, and distribute the MISMO
DTD and its accompanying documentation for any purpose and without
fee is hereby granted in perpetuity, provided that the above copyright
notice and this paragraph appear in all copies. &nbsp;&nbsp; The copyright holders make
no representation about the suitability of the DTD for any purpose.
""")
        outfh.write("</p>\n")
        outfh.write("<hr />\n")

        outfh.write("<p>\n")
        outfh.write('<a href="#A1">Alphabetical by Standard Name</a><br />\n')        
        outfh.write('<a href="#A2">Alphabetical by Short Name</a><br />\n') 
        outfh.write("</p>\n")
        outfh.write("<hr />\n")
 
        outfh.write('<p><a name="A1"></a> <b>Alphabetical by Standard Name</b>\n')        
        outfh.write("</p>\n")
        outfh.write("<hr />\n")
        #outfh.write("<table border=\"1\" width=\"100%\" summary=\"attributemapinfo\">\n")
        outfh.write("<table border=\"1\" summary=\"attributemapinfo\">\n")
        outfh.write(" <tr bgcolor=\"#a0b8c8\">\n")
        outfh.write("  <th align=\"center\">Standard Name</th>\n")
        outfh.write("  <th align=\"center\">Short Name</th>\n")
        outfh.write(" </tr>\n")
        self.attribute_name_list1.sort()
        itemcount = 0
        for item in self.attribute_name_list1:
            itemcount = itemcount + 1
            if operator.mod(itemcount,2) == 0:
                outfh.write('<tr bgcolor="WHITE">\n')
            else:
                outfh.write('<tr bgcolor="#EEEEEE">\n')            
            outfh.write("  <td align=\"right\"><font size=\"3\">%s</font></td>\n" % (item[0]))
            outfh.write("  <td align=\"left\"><font size=\"3\">%s</font></td>\n" % (item[1]))
            outfh.write(" </tr>\n")
        outfh.write("</table>\n")
        outfh.write("<hr />\n")
                    
        outfh.write('<p><a name="A2"></a> <b>Alphabetical by Short Name</b>\n')           
        outfh.write("</p>\n")
        outfh.write("<hr />\n")
        #outfh.write("<table border=\"1\" width=\"100%\" summary=\"attributemapinfo\">\n")
        outfh.write("<table border=\"1\" summary=\"attributemapinfo\">\n")
        outfh.write(" <tr bgcolor=\"#a0b8c8\">\n")
        outfh.write("  <th align=\"center\">Short Name</th>\n")
        outfh.write("  <th align=\"center\">Standard Name</th>\n")
        outfh.write(" </tr>\n")
        self.attribute_name_list2.sort()
        itemcount = 0
        for item in self.attribute_name_list2:
            itemcount = itemcount + 1
            if operator.mod(itemcount,2) == 0:
                outfh.write('<tr bgcolor="WHITE">\n')
            else:
                outfh.write('<tr bgcolor="#EEEEEE">\n')
            outfh.write("  <td align=\"right\"><font size=\"3\">%s</font></td>\n" % (item[0]))
            outfh.write("  <td align=\"left\"><font size=\"3\">%s</font></td>\n" % (item[1]))
            outfh.write(" </tr>\n")
        outfh.write("</table>\n")
        outfh.write("<hr />\n")

        outfh.write("</body>\n" )
        outfh.write("</html>\n" )

        # all done
        outfh.close()

    def containers_attr_style(self,LDD_rows,outfh):
        """ 
        process the containers to get to the data items
        """
        if self.debug > 1: sys.stderr.write( "processing containers for xslt map ...\n" )
        if self.debug > 1: sys.stderr.write( "%5i LDD Rows to process.\n" % len(LDD_rows) ) 
        row = LDD_rows[0] # extract the first row
        prev_container_name = row[2] # prime the prev name logic
        container_count = 0          # prime the container count
        containers = []
        container_rows = []
        i = 0
        while i < len(LDD_rows):
            row = LDD_rows[i]
            if prev_container_name == row[2] :
                container_rows.append(row)
                if self.debug > 1: sys.stderr.write("   same container, now appending subelement %s\n" % (row[4]) )
                self.logfh.write("   same container, now appending subelement %s\n" % (row[4]) )
            else :
                # handle end of a container
                if self.debug > 1: sys.stderr.write( "  Collected the rows , now processing %s container\n" % prev_container_name )
                self.logfh.write("  Collected the rows , now processing %s container\n" % (prev_container_name) )
                containers.append(prev_container_name) # store the container name
                prev_container_name = row[2] # reset the prev container name
                container_count += 1
                #
                self.write_container_attr(outfh,container_rows)
                # 07/31/03 removed the repeat for the ELEs, not necessary
                ### repeat to create/write the ele : added 02/15/2002 
                ##elefh= open('./projects/'+ self.project_name + '/output/ele/'+ container_rows[0][2]+ '.ele' ,"w") # open the elefile
                ##self.write_container_attr(elefh,container_rows,commentflag)
                ##elefh.close()
                #
                container_rows = [] # reset the
                container_rows.append(row)
                if self.debug > 1: sys.stderr.write("   new container, now appending subelement %s\n" % (row[4]) )
                self.logfh.write("   new container, now appending subelement %s\n" % (row[4]) )
            i += 1  # index to the next row in the LDD
            # if we've reached the end handle the last container
            #if self.debug > 1: sys.stderr.write("i is %i\n" % i )
            if i == len(LDD_rows):
                if self.debug > 1: sys.stderr.write( "  Collected the rows , now processing %s container\n" % prev_container_name )
                self.logfh.write("  Collected the rows , now pProcessing %s container\n" % (prev_container_name) )
                container_count += 1
                self.write_container_attr(outfh,container_rows)
                # 07/31/03 removed the repeat for the ELEs, not necessary
                ### repeat to create/write the ele : added 02/15/2002
                ##elefh= open('./projects/'+ self.project_name + '/output/ele/'+ container_rows[0][2]+ '.ele' ,"w") # open the elefile
                ##self.write_container_attr(elefh,container_rows,commentflag)
                ##elefh.close()
            #sys.stderr.write("%i bottom of loop\n" % i ) 
        sys.stderr.write( "%i containers and %i rows have been processed.\n" % (container_count,i) )
        self.logfh.write("%i containers and %i rows have been processed.\n" % (container_count,i))


    def write_container_attr(self,outfh,container_rows):
        """
        Process a container for the map
        """
        row = container_rows[0]
        #
##        dtdfh.write("<!-- ================================================================= -->\n")
##        dtdfh.write("<!--   Container: %-50s   -->\n" % ( row[2] ))
##        if len(row[1]) != 0:
##            dtdfh.write("<!-- =================================================================\n" )
##            # left = 0 center = 1 right = 2
##
##            formatter = TextFormatter.TextFormatter(
##                (
##                    {'width':17, 'margin': 0, 'alignment': 2},{'width':50, 'margin': 1, 'alignment': 0}
##                )
##            )
##            fout = formatter.compose(("Description: ",row[1])) 
##            dtdfh.write("%s" % (fout))
##            dtdfh.write("     ================================================================= -->\n" )
##        else:
##            dtdfh.write("<!-- ================================================================= -->\n" )
##        sys.stderr.write("  processing container %s ..." % (row[2]))
##        self.write_element(outfh,row)
##        #dtdfh.write("\n")
##        outfh.flush()
##        sys.stderr.write("done.\n")                      
        
        if self.debug > 1: sys.stderr.write("    in %s there are %i rows to process\n" % (row[2],(len(container_rows)+1)))                          
        i = 0
        j = 0
        while i < len(container_rows):
            row = container_rows[i]
            # 

            if row[6] == "Enumerated" or row[6] == "EnumeratedWithDefault":
                if self.debug > 1: sys.stderr.write("    getting enum value %s %i ..." %(row[4],i) )
                enum_string_value,j,enum_rows = genny_util.get_enum_string_value(i,container_rows,self.debug)
                if self.debug > 1: sys.stderr.write(" found enum str %s %i .\n" % (enum_string_value,j) )



##            # If the data item is an enumerated value then get the string
##            enum_string_value = ""
##            if row[6] == "Enumerated" or row[6] == "EnumeratedWithDefault":
##                if self.debug > 1: sys.stderr.write("    getting enum value %s %i ..." %(row[4],i) )
##                enum_string_value,j,enum_rows = genny_util.get_enum_string_value(i,container_rows,self.debug)
##                if self.debug > 1: sys.stderr.write(" found enum str %s %i .\n" % (enum_string_value,j) )
##            # Output the comments if we are asked to            
##            if commentflag != "NO":
##                if self.debug > 1: sys.stderr.write("    adding comments %s ...\n" % (row[4]) )
##                dtdfh.write("\n")
##                dtdfh.write("<!-- ================================================================= -->\n")
##                dtdfh.write(    "<!--   Data Item: %s -->\n" % (row[4]))
##
##                # word wrapped description
##                dtdfh.write(    "<!-- \n" )
##                # left = 0 center = 1 right = 2
##                formatter = TextFormatter.TextFormatter(
##                    (
##                        {'width':17, 'margin': 0, 'alignment': 2},{'width':50, 'margin': 1, 'alignment': 0}
##                    )
##                )
##                dtdfh.flush()
##                fout = formatter.compose(("Description: ",genny_util.xml_unescape(row[5]))) 
##                dtdfh.write("%s" % (fout))
##                dtdfh.write(" -->\n" )
##    ##          dtdfh.write(    "<!-- Description: %s -->\n" % (row[5]))
##
##                if enum_string_value == "":
##                    dtdfh.write("<!--    Datatype: %s -->\n" % (row[6]))
##                else:
##                    dtdfh.write("<!--    Datatype: Enumerated Value List -->\n" )
##                    for enum_row in enum_rows:
##                        dtdfh.write("<!-- Valid Value: %s -->\n" % (enum_row[0]))
##                        if len(enum_row[1]) != 0:
##    ##                      dtdfh.write("Desc: %s -->\n" % (enum_row[1]))
##                            # word wrapped description
##                            dtdfh.write(    "<!-- \n" )
##                            # left = 0 center = 1 right = 2
##                            formatter = TextFormatter.TextFormatter(
##                                (
##                                    {'width':17, 'margin': 0, 'alignment': 2},{'width':50, 'margin': 1, 'alignment': 0}
##                                )
##                            )
##                            fout = formatter.compose(("Value Desc: ",genny_util.xml_unescape(row[5]))) 
##                            dtdfh.write("%s" % (fout))
##                            dtdfh.write(" -->\n" )
##                            dtdfh.write("<!-- ================================================================= -->\n")
##                        else :
##                            dtdfh.write("<!-- ================================================================= -->\n")
            # Output the Attribute to the DTD
            #
            long_attribute_name = row[4]
            #print "row 40 %s" % row[40]
            short_attribute_name = genny_util.getShortName(row,i,debug=0)
            # don't add it twice
            if  self.attribute_name_list1.count([long_attribute_name,short_attribute_name]) == 0:
                self.attribute_name_list1.append([long_attribute_name,short_attribute_name])
            # don't add it to the list twice
            if  self.attribute_name_list2.count([short_attribute_name,long_attribute_name]) == 0:
                self.attribute_name_list2.append([short_attribute_name,long_attribute_name])
            
            matchtemplate = '''
<xsl:template match="%s/@%s">
  <xsl:attribute name="%s">
    <xsl:value-of select="."/>
  </xsl:attribute>
</xsl:template>
'''
            if self.maptypeflag == 'S2L':
                # Short to Long Map
                outfh.write(matchtemplate % (row[2],short_attribute_name,long_attribute_name))
            else:
                # Long to Short Map
                outfh.write(matchtemplate % (row[2],long_attribute_name,short_attribute_name))

            ##i = i +1

            if (row[6] == "Enumerated" or row[6] == "EnumeratedWithDefault") and j != 0:
                # skip the enumerated value rows that are passed in 
                i=i+j
                if self.debug > 1: sys.stderr.write("   skipping %i rows at %i.\n" % (j,i) )
                j=0
            else:
                i += 1




                
##            # actually output the row
##            if row[6] == "Enumerated":
##                dtdfh.write("<!ATTLIST %s %s ( %s ) #IMPLIED>\n" % (row[2],attribute_name,enum_string_value) )
##            # added 01/24/2002 to handle the enumerated with a default value
##            elif row[6] == "EnumeratedWithDefault":
##                dtdfh.write('<!ATTLIST %s %s ( %s ) "%s" >\n' % (row[2],attribute_name,enum_string_value,row[11]) )
##            # added 06/26/2001 to handle the Boolean
##            # a Boolean is defined as an enumerated list with Y/N as the possible values
##            #
##            elif row[6] == "Boolean":
##                dtdfh.write("<!ATTLIST %s %s ( true | false | 0 | 1 ) #IMPLIED>\n" % (row[2],attribute_name) )
##            # handle the ID datatype
##            elif row[6] == "ID":
##                dtdfh.write("<!ATTLIST %s %s ID  #IMPLIED>\n" % (row[2],attribute_name) )           
##            # handle the IDREF datatype
##            elif row[6] == "IDREF":
##                ##dtdfh.write("<!ATTLIST %s %s IDREF #IMPLIED>\n" % (row[2],(row[4]+"_Referencing"+row[10]+"ID")) )
##                m_refname = string.replace(row[10], "_", " ")
##                m_refname = mismoextensions.mismo_item_naming_scheme(m_refname,m_refname,self.logfh,"Y")
##                dtdfh.write("<!ATTLIST %s %s IDREF #IMPLIED>\n" % (row[2],(attribute_name +"_Referencing_"+ m_refname + "ID")))
##            # added 01/24/2002 to handle the IDREFS datatype 
##            elif row[6] == "IDREFS":
##                m_refname = string.replace(row[10], "_", " ")
##                m_refname = mismoextensions.mismo_item_naming_scheme(m_refname,m_refname,self.logfh,"Y")
##                dtdfh.write("<!ATTLIST %s %s IDREFS #IMPLIED>\n" % (row[2],(attribute_name +"_Referencing_"+ m_refname + "IDs")))
##            # handle the Fixed datatype
##            elif row[6] == "Fixed":
##                dtdfh.write("<!ATTLIST %s %s CDATA #FIXED '%s'>\n" % (row[2],attribute_name,row[11]) )
##            else:
##                # added 08-06-2002 to skip xmlns
##                if row[4] != 'xmlns':
##                    dtdfh.write("<!ATTLIST %s %s CDATA  #IMPLIED>\n" % (row[2],attribute_name) )
##            # skip to the next data item we should process                  
##            if (row[6] == "Enumerated" or row[6] == "EnumeratedWithDefault") and j != 0:
##                # skip the enumerated value rows that are passed in 
##                i=i+j
##                if self.debug > 1: sys.stderr.write("   skipping %i rows at %i.\n" % (j,i) )
##                j=0
##            else:
##                i += 1
##            #dtdfh.write("\n")
##        dtdfh.write("\n")


##    def write_element(self,dtdfh,row):
##        """
##        Output a DTD element for the given container using the enhanced LDD meta data
##        """
##        # output the element name
##        dtdfh.write('<!ELEMENT %-30s ' % (row[2]))
##        # output the content
##        if len(row[23]) == 0:
##            # EMPTY content
##            dtdfh.write(' EMPTY >\n')
##        elif row[23][0][0] == 'ANY':
##            # ANY content
##            dtdfh.write(' ANY >\n')
##        else:
##            dtdfh.write('(\n')
##            # otherwise this container has children containers
##            ele_child_count = 0
##            for child_c_list in row[23]:
##                # if this is not the first child and add a comma
##                ##print ("\n%s : %s : %s" % (child_c_list[0],child_c_list[1],child_c_list[2]))
##                if ele_child_count != 0:
##                    dtdfh.write(',\n')
##                ele_cardinality = '?'  # default to 0 or 1
##                # if min is zero and max is Unbounded then set to *
##                if child_c_list[1] == '0' and (child_c_list[2] == 'u'or child_c_list[2] =='U'):
##                    ele_cardinality = '*'
##                # if both min is 1 and max is U then set to 1 or more
##                elif child_c_list[1] == '1' and (child_c_list[2] == 'u' or child_c_list[2] =='U'):
##                    ele_cardinality = "+"
##                # otherwise if max unbounded set to zero or more    
##                elif child_c_list[2] == 'u'or child_c_list[2] =='U':
##                    ele_cardinality = '*'
##                # if both min and max are 1 then don't have any cardinality
##                # i.e. one and only one
##                elif child_c_list[1] == '1' and child_c_list[1] == '1':
##                    ele_cardinality = ""
##                ##print ("ele cardinality %s" % (ele_cardinality))    
##                # output the ELEMENT to the DTD
##                dtdfh.write('               ')
##                dtdfh.write('               ')
##                dtdfh.write('           ')
##                dtdfh.write(' %s%s' % (child_c_list[0],ele_cardinality))
##                ele_child_count = ele_child_count + 1
##            # close the tag
##            dtdfh.write('\n')
##            dtdfh.write('               ')
##            dtdfh.write('               ')
##            dtdfh.write('           ')
##            dtdfh.write(')>\n')
