#!/usr/bin/env/python2.3
"""
Utility to verify the loan count and balances
for a MISMO v2.x Servicing Transfer instance document
"""

import sys
import operator
import xml.sax

found_error = False

class FileHandler(xml.sax.ContentHandler):

  def __init__(self):
    self.getCont  = 0
    self.contents = ""
    self.filename = ""

    # counts calculated from the parsing of the
    # instance document
    self.found_PayeeTotalCount = 0
    self.found_InvestorTotalCount = 0
    self.found_PoolTotalCount = 0
    self.found_LoanTotalCount = 0

    self.found_UPBTotalAmount  = 0
    self.found_EscrowBalanceTotalAmount  = 0
    self.found_RemainingBuydownBalanceAmountTotalAmount  = 0
    self.found_DeferredInterestBalanceAmountTotalAmount  = 0
    self.found_RecoverableCorporateAdvanceFromBorrowerAmountTotalAmount  = 0
    self.found_RecoverableCorporateAdvanceFromThirdPartyAmountTotalAmount  = 0
    self.found_BankruptcySuspenseBalanceAmountTotalAmount  = 0
    self.found_SuspenseBalanceAmountTotalAmount  = 0
    self.found_RestrictedEscrowBalanceAmountTotalAmount  = 0
    self.found_EscrowCompletionFundsBalanceAmountTotalAmount  = 0
    self.found_UncollectedLateChargeBalanceAmountTotalAmount  = 0
    self.found_CurrentBuydownSubsidyAmountTotalAmount  = 0
    self.found_CurrentTaxAndInsurancePaymentAmountTotalAmount  = 0
    self.found_CurrentTotalOptionalProductsPaymentAmountTotalAmount  = 0
    self.found_CurrentPrincipalAndInterestPaymentAmountTotalAmount  = 0
    self.found_CurrentTotalPaymentAmountTotalAmount  = 0
    self.found_CurrentHUD235SubsidyPaymentAmountTotalAmount  = 0


    # counts included in the verification containers of the
    # instance document
    self.PayeeTotalCount = 0
    self.InvestorTotalCount = 0
    self.PoolTotalCount = 0
    self.LoanTotalCount = 0

    self.UPBTotalAmount  = 0
    self.EscrowBalanceTotalAmount  = 0
    self.RemainingBuydownBalanceAmountTotalAmount  = 0
    self.DeferredInterestBalanceAmountTotalAmount  = 0
    self.RecoverableCorporateAdvanceFromBorrowerAmountTotalAmount  = 0
    self.RecoverableCorporateAdvanceFromThirdPartyAmountTotalAmount  = 0
    self.BankruptcySuspenseBalanceAmountTotalAmount  = 0
    self.SuspenseBalanceAmountTotalAmount  = 0
    self.RestrictedEscrowBalanceAmountTotalAmount  = 0
    self.EscrowCompletionFundsBalanceAmountTotalAmount  = 0
    self.UncollectedLateChargeBalanceAmountTotalAmount  = 0
    self.CurrentBuydownSubsidyAmountTotalAmount  = 0
    self.CurrentTaxAndInsurancePaymentAmountTotalAmount  = 0
    self.CurrentTotalOptionalProductsPaymentAmountTotalAmount  = 0
    self.CurrentPrincipalAndInterestPaymentAmountTotalAmount  = 0
    self.CurrentTotalPaymentAmountTotalAmount  = 0
    self.CurrentHUD235SubsidyPaymentAmountTotalAmount  = 0

  def startElement(self, name, attrs):
      # section for each ELEMENT found
      #
      if name == "LOAN":
          self.found_LoanTotalCount +=1
          if operator.mod(self.found_LoanTotalCount,100) == 0:
              sys.stderr.write("current Loan count is %i\n" % (self.found_LoanTotalCount) )
          try :
              self.found_UPBTotalAmount += int(attrs.get('UPBAmount', ""))
          except:
              pass
          try :
              self.found_EscrowBalanceTotalAmount += int(attrs.get('EscrowBalanceAmount', ""))
          except:
              pass
          try :
              self.found_RemainingBuydownBalanceAmountTotalAmount += int(attrs.get('RemainingBuydownBalanceAmount', ""))
          except:
              pass
          try :
              self.found_DeferredInterestBalanceAmountTotalAmount += int(attrs.get('DeferredInterestBalanceAmount', ""))
          except:
              pass
          try :
              self.found_RecoverableCorporateAdvanceFromBorrowerAmountTotalAmount += int(attrs.get('RecoverableCorporateAdvanceFromBorrowerAmount', ""))
          except:
              pass
          try :
              self.found_RecoverableCorporateAdvanceFromThirdPartyAmountTotalAmount += int(attrs.get('RecoverableCorporateAdvanceFromThirdPartyAmount', ""))
          except:
              pass
          try :
              self.found_BankruptcySuspenseBalanceAmountTotalAmount += int(attrs.get('BankruptcySuspenseBalanceAmount', ""))
          except:
              pass
          try :
              self.found_SuspenseBalanceAmountTotalAmount += int(attrs.get('SuspenseBalanceAmount', ""))
          except:
              pass
          try :
              self.found_RestrictedEscrowBalanceAmountTotalAmount += int(attrs.get('RestrictedEscrowBalanceAmount', ""))
          except:
              pass
          try :
              self.found_EscrowCompletionFundsBalanceAmountTotalAmount += int(attrs.get('EscrowCompletionFundsBalanceAmount', ""))
          except:
              pass
          try :
              self.found_UncollectedLateChargeBalanceAmountTotalAmount += int(attrs.get('UncollectedLateChargeBalanceAmount', ""))
          except:
              pass
          try :
              self.found_CurrentBuydownSubsidyAmountTotalAmount += int(attrs.get('CurrentBuydownSubsidyAmount', ""))
          except:
              pass
          try :
              self.found_CurrentTaxAndInsurancePaymentAmountTotalAmount += int(attrs.get('CurrentTaxAndInsurancePaymentAmount', ""))
          except:
              pass
          try :
              self.found_CurrentTotalOptionalProductsPaymentAmountTotalAmount += int(attrs.get('CurrentTotalOptionalProductsPaymentAmount', ""))
          except:
              pass
          try :
              self.found_CurrentPrincipalAndInterestPaymentAmountTotalAmount += int(attrs.get('CurrentPrincipalAndInterestPaymentAmount', ""))
          except:
              pass
          try :
              self.found_CurrentTotalPaymentAmountTotalAmount += int(attrs.get('CurrentTotalPaymentAmount', ""))
          except:
              pass
          try :
              self.found_CurrentHUD235SubsidyPaymentAmountTotalAmount += int(attrs.get('CurrentHUD235SubsidyPaymentAmount', ""))
          except:
              pass


      elif name == "PAYEE":
          self.found_PayeeTotalCount +=1
          if operator.mod(self.found_PayeeTotalCount,100) == 0:
              sys.stderr.write("current Payee count is %i\n" % (self.found_PayeeTotalCount) )

      elif name == "INVESTOR":
          self.found_InvestorTotalCount +=1
          if operator.mod(self.found_InvestorTotalCount,100) == 0:
              sys.stderr.write("current Investor count is %i\n" % (self.found_InvestorTotalCount) )

      elif name == "POOL":
          self.found_PoolTotalCount +=1
          if operator.mod(self.found_PoolTotalCount,100) == 0:
              sys.stderr.write("current Pool count is %i\n" % (self.found_PoolTotalCount) )

      elif name == "VERIFICATIONDATA_PAYEE":
          try :
              self.PayeeTotalCount = int(attrs.get('PayeeTotalCount', ""))
          except:
              self.PayeeTotalCount = 0

      elif name == "VERIFICATIONDATA_INVESTOR":
          try :
              self.InvestorTotalCount = int(attrs.get('InvestorTotalCount', ""))
          except:
              self.InvestorTotalCount = 0

      elif name == "VERIFICATIONDATA_POOL":
          try :
              self.PoolTotalCount = int(attrs.get('PoolTotalCount', ""))
          except:
              self.PoolTotalCount = 0

      elif name == "VERIFICATIONDATA_LOAN":
          try :
              self.LoanTotalCount = int(attrs.get('LoanTotalCount', ""))
          except:
              self.LoanTotalCount = 0

          try :
              self.UPBTotalAmount = int(attrs.get('UPBTotalAmount', ""))
          except:
              self.UPBTotalAmount = 0
          try :
              self.EscrowBalanceTotalAmount = int(attrs.get('EscrowBalanceTotalAmount', ""))
          except:
              self.EscrowBalanceTotalAmount = 0
          try :
              self.RemainingBuydownBalanceAmountTotalAmount = int(attrs.get('RemainingBuydownBalanceAmountTotalAmount', ""))
          except:
              self.RemainingBuydownBalanceAmountTotalAmount = 0
          try :
              self.DeferredInterestBalanceAmountTotalAmount = int(attrs.get('DeferredInterestBalanceAmountTotalAmount', ""))
          except:
              self.DeferredInterestBalanceAmountTotalAmount = 0
          try :
              self.RecoverableCorporateAdvanceFromBorrowerAmountTotalAmount = int(attrs.get('RecoverableCorporateAdvanceFromBorrowerAmountTotalAmount', ""))
          except:
              self.RecoverableCorporateAdvanceFromBorrowerAmountTotalAmount = 0
          try :
              self.RecoverableCorporateAdvanceFromThirdPartyAmountTotalAmount = int(attrs.get('RecoverableCorporateAdvanceFromThirdPartyAmountTotalAmount', ""))
          except:
              self.RecoverableCorporateAdvanceFromThirdPartyAmountTotalAmount = 0
          try :
              self.BankruptcySuspenseBalanceAmountTotalAmount = int(attrs.get('BankruptcySuspenseBalanceAmountTotalAmount', ""))
          except:
              self.BankruptcySuspenseBalanceAmountTotalAmount = 0
          try :
              self.SuspenseBalanceAmountTotalAmount = int(attrs.get('SuspenseBalanceAmountTotalAmount', ""))
          except:
              self.SuspenseBalanceAmountTotalAmount = 0
          try :
              self.RestrictedEscrowBalanceAmountTotalAmount = int(attrs.get('RestrictedEscrowBalanceAmountTotalAmount', ""))
          except:
              self.RestrictedEscrowBalanceAmountTotalAmount = 0
          try :
              self.EscrowCompletionFundsBalanceAmountTotalAmount = int(attrs.get('EscrowCompletionFundsBalanceAmountTotalAmount', ""))
          except:
              self.EscrowCompletionFundsBalanceAmountTotalAmount = 0
          try :
              self.UncollectedLateChargeBalanceAmountTotalAmount = int(attrs.get('UncollectedLateChargeBalanceAmountTotalAmount', ""))
          except:
              self.UncollectedLateChargeBalanceAmountTotalAmount = 0
          try :
              self.CurrentBuydownSubsidyAmountTotalAmount = int(attrs.get('CurrentBuydownSubsidyAmountTotalAmount', ""))
          except:
              self.CurrentBuydownSubsidyAmountTotalAmount = 0
          try :
              self.CurrentTaxAndInsurancePaymentAmountTotalAmount = int(attrs.get('CurrentTaxAndInsurancePaymentAmountTotalAmount', ""))
          except:
              self.CurrentTaxAndInsurancePaymentAmountTotalAmount = 0
          try :
              self.CurrentTotalOptionalProductsPaymentAmountTotalAmount = int(attrs.get('CurrentTotalOptionalProductsPaymentAmountTotalAmount', ""))
          except:
              self.CurrentTotalOptionalProductsPaymentAmountTotalAmount = 0
          try :
              self.CurrentPrincipalAndInterestPaymentAmountTotalAmount = int(attrs.get('CurrentPrincipalAndInterestPaymentAmountTotalAmount', ""))
          except:
              self.CurrentPrincipalAndInterestPaymentAmountTotalAmount = 0
          try :
              self.CurrentTotalPaymentAmountTotalAmount = int(attrs.get('CurrentTotalPaymentAmountTotalAmount', ""))
          except:
              self.CurrentTotalPaymentAmountTotalAmount = 0
          try :
              self.CurrentHUD235SubsidyPaymentAmountTotalAmount = int(attrs.get('CurrentHUD235SubsidyPaymentAmountTotalAmount', ""))
          except:
              self.CurrentHUD235SubsidyPaymentAmountTotalAmount = 0
      else:
         # if it's any other ELEMENT then skip
         pass

  def characters(self, ch):
      if (self.getCont):
          self.contents += ch

  def endElement(self, name):
      pass

def check_count(found_cnt,given_cnt,type):
    correct_cnt_msg = "MATCH: %s cnt of %i matches the file total.\n"
    error_cnt_msg   = "ERROR: %s cnt of %i DOES NOT match file total %i.\n"

    if found_cnt == given_cnt:
        sys.stderr.write(correct_cnt_msg % (type,given_cnt) )
    else:
        sys.stderr.write(error_cnt_msg % (type,given_cnt,found_cnt) )
        found_error = True

def check_amt(found_amt,given_amt,type):
    correct_amt_msg = "MATCH: %s of $%.2f matches the file total.\n"
    error_amt_msg   = "ERROR: %s of $%.2f DOES NOT match file total $%.2f.\n"

    if found_amt == given_amt:
        sys.stderr.write(correct_amt_msg % (type,given_amt) )
    else:
        sys.stderr.write(error_amt_msg % (type,given_amt,found_amt) )
        found_error = True
        
# =====================================================================
if __name__ == '__main__':
    # parse the instance document
    fh = FileHandler()
    parser = xml.sax.make_parser()
    parser.setContentHandler(fh)
    parser.parse(sys.stdin)
    # verify/check the Counts
    sys.stderr.write('\n\n')
    sys.stderr.write('#### Results of Count Comparison\n')
    check_count(fh.found_PayeeTotalCount,fh.PayeeTotalCount,'Payee')
    check_count(fh.found_InvestorTotalCount,fh.InvestorTotalCount,'Investor')
    check_count(fh.found_PoolTotalCount,fh.PoolTotalCount,'Pool')
    check_count(fh.found_LoanTotalCount,fh.LoanTotalCount,'Loan')
    # verify/check the Balance Totals
    sys.stderr.write('\n\n')
    sys.stderr.write('#### Results of Amount Comparison\n')
    check_amt(fh.found_UPBTotalAmount,fh.UPBTotalAmount,'UPB Total Amount')
    check_amt(fh.found_EscrowBalanceTotalAmount,fh.EscrowBalanceTotalAmount,'Escrow Balance Total Amount')
    check_amt(fh.found_RemainingBuydownBalanceAmountTotalAmount,fh.RemainingBuydownBalanceAmountTotalAmount,'Remaining Buydown Balance Amount Total Amount')
    check_amt(fh.found_DeferredInterestBalanceAmountTotalAmount,fh.DeferredInterestBalanceAmountTotalAmount,'Deferred Interest Balance Amount Total Amount')
    check_amt(fh.found_RecoverableCorporateAdvanceFromBorrowerAmountTotalAmount,fh.RecoverableCorporateAdvanceFromBorrowerAmountTotalAmount,'Recoverable Corporate Advance From Borrower Amount Total Amount')
    check_amt(fh.found_RecoverableCorporateAdvanceFromThirdPartyAmountTotalAmount,fh.RecoverableCorporateAdvanceFromThirdPartyAmountTotalAmount,'Recoverable Corporate Advance From Third Party Amount Total Amount')
    check_amt(fh.found_BankruptcySuspenseBalanceAmountTotalAmount,fh.BankruptcySuspenseBalanceAmountTotalAmount,'Bankruptcy Suspense Balance Amount Total Amount')
    check_amt(fh.found_SuspenseBalanceAmountTotalAmount,fh.SuspenseBalanceAmountTotalAmount,'Suspense Balance Amount Total Amount')
    check_amt(fh.found_RestrictedEscrowBalanceAmountTotalAmount,fh.RestrictedEscrowBalanceAmountTotalAmount,'Restricted Escrow Balance Amount Total Amount')
    check_amt(fh.found_EscrowCompletionFundsBalanceAmountTotalAmount,fh.EscrowCompletionFundsBalanceAmountTotalAmount,'Escrow Completion Funds Balance Amount Total Amount')
    check_amt(fh.found_UncollectedLateChargeBalanceAmountTotalAmount,fh.UncollectedLateChargeBalanceAmountTotalAmount,'Uncollected Late Charge Balance Amount Total Amount')
    check_amt(fh.found_CurrentBuydownSubsidyAmountTotalAmount,fh.CurrentBuydownSubsidyAmountTotalAmount,'Current Buydown Subsidy Amount Total Amount')
    check_amt(fh.found_CurrentTaxAndInsurancePaymentAmountTotalAmount,fh.CurrentTaxAndInsurancePaymentAmountTotalAmount,'Current Tax And Insurance Payment Amount Total Amount')
    check_amt(fh.found_CurrentTotalOptionalProductsPaymentAmountTotalAmount,fh.CurrentTotalOptionalProductsPaymentAmountTotalAmount,'Current Total Optional Products Payment Amount Total Amount')
    check_amt(fh.found_CurrentTotalPaymentAmountTotalAmount,fh.CurrentTotalPaymentAmountTotalAmount,'Current Total Payment Amount Total Amount')
    check_amt(fh.found_CurrentHUD235SubsidyPaymentAmountTotalAmount,fh.CurrentHUD235SubsidyPaymentAmountTotalAmount,'Current HUD235 Subsidy Payment Amount Total Amount')
    sys.stderr.write('\n\n')
    
    if found_error:
      sys.exit(1)
    else:
      sys.exit(0)